# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
SMS composer
'''

import wx
import wx.lib.editor.editor
import wx.lib.mixins.listctrl
import Wammu
import Wammu.Data
import Wammu.MessageDisplay
import Wammu.Utils
import Wammu.PhoneValidator
import Wammu.Select
import Wammu.EditContactList
if Wammu.gammu_error is None:
    import gammu
import Wammu.Locales
from Wammu.Locales import ugettext as _

class MessagePreview(wx.Dialog):
    text = '''
<html>
<body>
%s
<center>
<p><wxp module="wx" class="Button">
    <param name="id"    value="ID_OK">
</wxp></p>
</center>
</body>
</html>
'''

    def __init__(self, parent, content):
        wx.Dialog.__init__(self, parent, -1, _('Message preview'))
        html = wx.html.HtmlWindow(self, -1, size=(420, -1))
        html.SetPage(self.text % content)
        btn = html.FindWindowById(wx.ID_OK)
        btn.SetDefault()
        ir = html.GetInternalRepresentation()
        html.SetSize((ir.GetWidth() + 25, ir.GetHeight() + 25))
        self.SetClientSize(html.GetSize())
        self.CentreOnParent(wx.BOTH)

class StyleEdit(wx.Dialog):
    def __init__(self, parent, entry):
        wx.Dialog.__init__(self, parent, -1, _('Text style'))

        self.sizer = wx.GridBagSizer()

        self.entry = entry

        self.fmt = {}

        row = 1

        col = 1
        maxcol = 1

        for x in Wammu.Data.TextFormats:
            if len(x) == 2:
                name = x[1][0]
                text = x[1][1]
                self.fmt[name] = wx.CheckBox(self, -1, text)
                if name in self.entry:
                    self.fmt[name].SetValue(self.entry[name])
                self.sizer.Add(self.fmt[name], pos=(row, col))
                col = col + 2
            else:
                if col > 1:
                    row = row + 2
                    maxcol = max(col, maxcol)
                    col = 1

                self.sizer.Add(wx.StaticText(self, -1, x[0][0] + ':'), pos=(row, col))
                col = col + 2

                rb = wx.RadioButton(self, -1, x[0][1], style=wx.RB_GROUP)
                rb.SetValue(True)
                self.sizer.Add(rb, pos=(row, col))
                col = col + 2
                for name, text, fmt in x[1:]:
                    self.fmt[name] = wx.RadioButton(self, -1, text)
                    if name in self.entry:
                        self.fmt[name].SetValue(self.entry[name])
                    self.sizer.Add(self.fmt[name], pos=(row, col))
                    col = col + 2
                row = row + 2
                maxcol = max(col, maxcol)
                col = 1

        if col > 1:
            row = row + 2
            maxcol = max(col, maxcol)
            col = 1

        self.ok = wx.Button(self, wx.ID_OK)
        self.sizer.Add(self.ok, pos=(row, 1), span=wx.GBSpan(colspan=maxcol), flag=wx.ALIGN_CENTER)
        wx.EVT_BUTTON(self, wx.ID_OK, self.Okay)

        self.sizer.AddSpacer((5, 5), pos=(row + 1, maxcol + 1))

        self.sizer.Fit(self)
        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)

        self.CentreOnParent(wx.BOTH)

    def Okay(self, evt):
        for x in Wammu.Data.TextFormats:
            for name, text, fmt in x[1:]:
                self.entry[name] = self.fmt[name].GetValue()
        self.EndModal(wx.ID_OK)

class AutoSizeList(wx.ListCtrl, wx.lib.mixins.listctrl.ListCtrlAutoWidthMixin):
    def __init__(self, parent, firstcol):
        wx.ListCtrl.__init__(self, parent, -1, style=wx.LC_REPORT | wx.LC_HRULES | wx.LC_VRULES | wx.LC_SINGLE_SEL | wx.SUNKEN_BORDER, size=(-1, 100))
        self.InsertColumn(0, firstcol)
        wx.lib.mixins.listctrl.ListCtrlAutoWidthMixin.__init__(self)

class GenericEditor(wx.Panel):
    """
    Generic class for static text with some edit control.
    """
    def __init__(self, parent, part, cfg, use_unicode):
        wx.Panel.__init__(self, parent, -1, style=wx.RAISED_BORDER)
        self.part = part
        self.cfg = cfg
        self.use_unicode = use_unicode

class TextEditor(GenericEditor):
    def __init__(self, parent, cfg, part, use_unicode):
        GenericEditor.__init__(self, parent, cfg, part, use_unicode)

        self.backuptext = ''

        self.sizer = wx.GridBagSizer()

        self.edit = wx.TextCtrl(self, -1, style=wx.TE_MULTILINE)

        self.sizer.Add(self.edit, pos=(0, 0), flag=wx.EXPAND, span=wx.GBSpan(colspan=4))
        self.sizer.AddGrowableCol(1)
        self.sizer.AddGrowableCol(2)
        self.sizer.AddGrowableRow(0)

        self.concat = wx.CheckBox(self, -1, _('Concatenated'))
        self.concat.SetToolTipString(_('Create concatenated message, what allows to send longer messages.'))
        self.concat.SetValue(self.part['ID'] != 'Text')
        wx.EVT_CHECKBOX(self.concat, self.concat.GetId(), self.OnConcatChange)
        self.sizer.Add(self.concat, pos=(1, 0), flag=wx.ALIGN_CENTER_VERTICAL)

        self.leninfo = wx.StaticText(self, -1, '')
        self.sizer.Add(self.leninfo, pos=(1, 3), flag=wx.ALIGN_RIGHT)

        self.stylebut = wx.Button(self, -1, _('Style'))
        wx.EVT_BUTTON(self, self.stylebut.GetId(), self.StylePressed)
        self.sizer.Add(self.stylebut, pos=(1, 1), flag=wx.ALIGN_CENTER)

        self.OnConcatChange()

        wx.EVT_TEXT(self.edit, self.edit.GetId(), self.TextChanged)
        if 'Buffer' in self.part:
            self.edit.SetValue(self.part['Buffer'])

        self.sizer.Fit(self)
        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)
        self.TextChanged()

    def OnUnicode(self, newu):
        self.use_unicode = newu
        self.CheckTextLen()

    def OnConcatChange(self, evt=None):
        self.stylebut.Enable(self.concat.GetValue())
        self.CheckTextLen()

    def CheckTextLen(self, evt=None):
        if not self.concat.GetValue():
            if self.use_unicode:
                self.edit.SetValue(self.edit.GetValue()[:70])
            else:
                self.edit.SetValue(self.edit.GetValue()[:160])

    def StylePressed(self, evt):
        dlg = StyleEdit(self, self.part)
        dlg.ShowModal()
        del dlg

    def TextChanged(self, evt=None):
        txt = self.edit.GetValue()
        length = len(txt)
        if not self.concat.GetValue() and ((self.use_unicode and length > 70) or (not self.use_unicode and length > 160)):
            self.edit.SetValue(self.backuptext)
            return
        length = len(self.edit.GetValue())
        self.leninfo.SetLabel(Wammu.Locales.ngettext('%d char', '%d chars', length) % length)
        self.sizer.Layout()
        self.backuptext = txt

    def GetValue(self):
        if self.concat.GetValue():
            if self.cfg.Read('/Message/16bitId') == 'yes':
                self.part['ID'] = 'ConcatenatedTextLong16bit'
            else:
                self.part['ID'] = 'ConcatenatedTextLong'
        else:
            self.part['ID'] = 'Text'
        self.part['Buffer'] = Wammu.Locales.UnicodeConv(self.edit.GetValue())
        return self.part

class PredefinedAnimEditor(GenericEditor):
    def __init__(self, parent, part, cfg, use_unicode):
        GenericEditor.__init__(self, parent, part, cfg, use_unicode)
        self.sizer = wx.GridBagSizer()

        values = []
        for x in Wammu.Data.PredefinedAnimations:
            values.append(x[0])

        self.sizer.AddGrowableRow(0)
        self.sizer.AddGrowableCol(0)
        self.sizer.AddGrowableCol(1)
        self.sizer.AddGrowableCol(2)

        self.edit = wx.Choice(self, -1, choices=values)

        bitmap = wx.BitmapFromXPMData(Wammu.Data.UnknownPredefined)
        self.bitmap = wx.StaticBitmap(self, -1, bitmap, (0, 0))

        wx.EVT_CHOICE(self.edit, self.edit.GetId(), self.OnChange)

        if 'Number' not in self.part:
            self.part['Number'] = 0

        self.edit.SetSelection(self.part['Number'])
        self.OnChange()

        self.sizer.Add(wx.StaticText(self, -1, _('Select predefined animation:')), pos=(0, 0), flag=wx.ALIGN_CENTER_VERTICAL)
        self.sizer.Add(self.edit, pos=(0, 1), flag=wx.ALIGN_CENTER)
        self.sizer.Add(self.bitmap, pos=(0, 2), flag=wx.ALIGN_CENTER)

        self.sizer.Fit(self)
        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)

    def OnChange(self, evt=None):
        bitmap = wx.BitmapFromXPMData(Wammu.Data.PredefinedAnimations[self.edit.GetSelection()][1])
        self.bitmap.SetBitmap(bitmap)

    def GetValue(self):
        self.part['ID'] = 'EMSPredefinedAnimation'
        self.part['Number'] = self.edit.GetSelection()
        return self.part

class PredefinedSoundEditor(GenericEditor):
    def __init__(self, parent, part, cfg, use_unicode):
        GenericEditor.__init__(self, parent, part, cfg, use_unicode)
        self.sizer = wx.GridBagSizer()

        values = []
        for x in Wammu.Data.PredefinedSounds:
            values.append(x[0])

        self.sizer.AddGrowableRow(0)
        self.sizer.AddGrowableCol(0)
        self.sizer.AddGrowableCol(1)

        self.edit = wx.Choice(self, -1, choices=values)

        if 'Number' not in self.part:
            self.part['Number'] = 0

        self.edit.SetSelection(self.part['Number'])

        self.sizer.Add(wx.StaticText(self, -1, _('Select predefined sound:')), pos=(0, 0), flag=wx.ALIGN_CENTER_VERTICAL)
        self.sizer.Add(self.edit, pos=(0, 1), flag=wx.ALIGN_CENTER)

        self.sizer.Fit(self)
        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)

    def GetValue(self):
        self.part['ID'] = 'EMSPredefinedSound'
        self.part['Number'] = self.edit.GetSelection()
        return self.part

# We should support more types...
#   ID, display text, match types, editor, init type
SMSParts = [
    (0, _('Text'), Wammu.Data.SMSIDs['Text'], TextEditor, 'ConcatenatedTextLong16bit'),
    (1, _('Predefined animation'), Wammu.Data.SMSIDs['PredefinedAnimation'], PredefinedAnimEditor, 'EMSPredefinedAnimation'),
    (2, _('Predefined sound'), Wammu.Data.SMSIDs['PredefinedSound'], PredefinedSoundEditor, 'EMSPredefinedSound'),
]

class SMSComposer(wx.Dialog):
    def __init__(self, parent, cfg, entry, values, action='save', addtext=True):
        wx.Dialog.__init__(self, parent, -1, _('Composing SMS'), style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.entry = entry
        self.cfg = cfg
        self.values = values
        if 'SMSInfo' not in entry:
            entry['SMSInfo'] = {}
            entry['SMSInfo']['Entries'] = []
            if self.cfg.Read('/Message/Concatenated') == 'yes':
                if self.cfg.Read('/Message/16bitId') == 'yes':
                    typ = 'ConcatenatedTextLong16bit'
                else:
                    typ = 'ConcatenatedTextLong'
            else:
                typ = 'Text'
            if 'Text' in entry:
                entry['SMSInfo']['Entries'].append({'ID': typ, 'Buffer': entry['Text']})
            elif addtext:
                entry['SMSInfo']['Entries'].append({'ID': typ, 'Buffer': ''})
        if 'Number' not in entry:
            entry['Number'] = ''

        self.sizer = wx.GridBagSizer()


        row = 1

        if action not in ['send', 'save']:
            action = 'save'

        self.send = wx.CheckBox(self, -1, _('Send message'))
        self.send.SetToolTipString(_('When checked, message is sent to recipient.'))
        self.send.SetValue(action == 'send')

        self.save = wx.CheckBox(self, -1, _('Save into folder'))
        self.save.SetToolTipString(_('When checked, message is saved to phone.'))
        self.save.SetValue(action == 'save')

        self.Bind(wx.EVT_CHECKBOX, self.OnSave, self.save)
        self.Bind(wx.EVT_CHECKBOX, self.OnSend, self.send)

        self.folder = wx.SpinCtrl(self, -1, '2', style=wx.SP_WRAP | wx.SP_ARROW_KEYS, min=0, max=100, initial=2)

        self.sizer.Add(self.send, pos=(row, 1), flag=wx.ALIGN_LEFT)
        self.sizer.Add(self.save, pos=(row, 6), flag=wx.ALIGN_LEFT)
        self.sizer.Add(self.folder, pos=(row, 7), flag=wx.ALIGN_LEFT)

        row = row + 2

        self.number = wx.TextCtrl(self, -1, entry['Number'], validator=Wammu.PhoneValidator.PhoneValidator(multi=True), size=(150, -1))
        self.contbut = wx.Button(self, -1, _('Add'))
        self.contbut.SetToolTipString(_('Add number of recipient from contacts.'))
        self.editlistbut = wx.Button(self, wx.ID_EDIT, _('Edit'))
        self.editlistbut.SetToolTipString(_('Edit recipients list.'))

        self.sizer.Add(wx.StaticText(self, -1, _('Recipient\'s numbers:')), pos=(row, 1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
        self.sizer.Add(self.number, pos=(row, 2), flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND, span=wx.GBSpan(colspan=4))
        self.sizer.Add(self.editlistbut, pos=(row, 6), flag=wx.ALIGN_CENTER)
        self.sizer.Add(self.contbut, pos=(row, 7), flag=wx.ALIGN_CENTER)

        self.Bind(wx.EVT_BUTTON, self.ContactPressed, self.contbut)
        self.Bind(wx.EVT_BUTTON, self.EditContactPressed, self.editlistbut)

        row = row + 2

        self.unicode = wx.CheckBox(self, -1, _('Unicode'))
        self.unicode.SetToolTipString(_('Unicode messages can contain national and other special characters, check this if you use non latin-1 characters. Your messages will require more space, so you can write less characters into single message.'))
        if 'Unicode' in self.entry:
            self.unicode.SetValue(self.entry['Unicode'])
        else:
            self.unicode.SetValue(self.cfg.Read('/Message/Unicode') == 'yes')

        self.sizer.Add(self.unicode, pos=(row, 1), flag=wx.ALIGN_LEFT)

        self.Bind(wx.EVT_CHECKBOX, self.OnUnicode, self.unicode)

        self.report = wx.CheckBox(self, -1, _('Delivery report'))
        self.report.SetToolTipString(_('Check to request delivery report for message.'))
        self.report.SetValue(self.cfg.Read('/Message/DeliveryReport') == 'yes')
        self.sizer.Add(self.report, pos=(row, 2), flag=wx.ALIGN_LEFT)

        self.sent = wx.CheckBox(self, -1, _('Sent'))
        self.sent.SetToolTipString(_('Check to save message as sent (has only effect when only saving message).'))
        self.sizer.Add(self.sent, pos=(row, 4), flag=wx.ALIGN_LEFT)

        self.flash = wx.CheckBox(self, -1, _('Flash'))
        self.flash.SetToolTipString(_('Send flash message - it will be just displayed on display, but not saved in phone.'))
        self.sizer.Add(self.flash, pos=(row, 6), flag=wx.ALIGN_LEFT)


        row = row + 2
        self.sizer.SetRows(row + 1)
        self.sizer.AddGrowableRow(row)

        self.current = AutoSizeList(self, _('Parts of current message'))
        self.available = AutoSizeList(self, _('Available message parts'))
        # FIXME: add icons?

        self.addbut = wx.Button(self, wx.ID_ADD)
        self.delbut = wx.Button(self, wx.ID_REMOVE)

        self.Bind(wx.EVT_BUTTON, self.AddPressed, self.addbut)
        self.Bind(wx.EVT_BUTTON, self.DeletePressed, self.delbut)

        self.sizer.Add(self.current, pos=(row, 1), flag=wx.EXPAND, span=wx.GBSpan(colspan=2, rowspan=2))
        self.sizer.Add(self.addbut, pos=(row, 4), flag=wx.ALIGN_CENTER)
        self.sizer.Add(self.delbut, pos=(row + 1, 4), flag=wx.ALIGN_CENTER)
        self.sizer.Add(self.available, pos=(row, 6), flag=wx.EXPAND, span=wx.GBSpan(colspan=2, rowspan=2))

        row = row + 3

        self.upbut = wx.Button(self, wx.ID_UP)
        self.dnbut = wx.Button(self, wx.ID_DOWN)

        self.sizer.Add(self.upbut, pos=(row, 1), flag=wx.ALIGN_CENTER)
        self.sizer.Add(self.dnbut, pos=(row, 2), flag=wx.ALIGN_CENTER)

        self.Bind(wx.EVT_BUTTON, self.UpPressed, self.upbut)
        self.Bind(wx.EVT_BUTTON, self.DnPressed, self.dnbut)

        row = row + 2
        self.sizer.SetRows(row + 1)
        self.sizer.AddGrowableRow(row)
        self.editorrow = row

        self.editor = wx.StaticText(self, -1, _('Create new message by adding part to left list...'), size=(-1, 150))
        self.sizer.Add(self.editor, pos=(row, 1), flag=wx.EXPAND, span=wx.GBSpan(colspan=7))

        row = row + 2

        self.preview = wx.Button(self, -1, _('Preview'))
        self.button_sizer = wx.StdDialogButtonSizer()
        self.button_sizer.AddButton(wx.Button(self, wx.ID_OK))
        self.button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        self.button_sizer.SetNegativeButton(self.preview)
        self.button_sizer.Realize()
        self.sizer.Add(self.button_sizer, pos=(row, 1), span=wx.GBSpan(colspan=7), flag=wx.ALIGN_RIGHT)

        self.Bind(wx.EVT_BUTTON, self.Okay, id=wx.ID_OK)
        self.Bind(wx.EVT_BUTTON, self.Preview, self.preview)

        self.sizer.AddSpacer((5, 5), pos=(row + 1, 8))
        self.sizer.AddGrowableCol(1)
        self.sizer.AddGrowableCol(2)
        self.sizer.AddGrowableCol(6)
        self.sizer.AddGrowableCol(7)
        self.sizer.Fit(self)
        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)

        self.prevedit = -1
        self.availsel = -1

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.CurrentSelected, self.current)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.AvailableSelected, self.available)

        if action == 'send':
            self.OnSave()
        else:
            self.OnSend()

        for x in SMSParts:
            self.available.InsertImageStringItem(x[0], x[1], x[0])
        self.available.SetItemState(0, wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED, wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED)

        self.GenerateCurrent()

    def ContactPressed(self, evt):
        v = Wammu.Select.SelectNumber(self, [] + self.values['contact']['ME'] + self.values['contact']['SM'])
        if v is not None:
            self.number.SetValue(self.number.GetValue() + ' ' + v)

    def EditContactPressed(self, evt):
        '''
        Opens dialog for editing contact list.
        '''
        contacts = [] + self.values['contact']['ME'] + self.values['contact']['SM']
        current = self.number.GetValue()
        dlg = Wammu.EditContactList.EditContactList(self, contacts, current)
        if dlg.ShowModal() == wx.ID_OK:
            self.number.SetValue(dlg.GetNumbers())

    def OnSend(self, evt=None):
        self.save.Enable(self.send.GetValue())
        self.number.GetValidator().empty = not self.send.GetValue()

    def OnSave(self, evt=None):
        self.send.Enable(self.save.GetValue())

    def GenerateCurrent(self, select=0):
        self.current.DeleteAllItems()
        for i in range(len(self.entry['SMSInfo']['Entries'])):
            found = False
            x = self.entry['SMSInfo']['Entries'][i]
            for p in SMSParts:
                if x['ID'] in p[2]:
                    self.current.InsertImageStringItem(i, p[1], p[0])
                    found = True
                    break
            if not found:
                self.current.InsertImageStringItem(i, _('Not supported id: %s') % x['ID'], -1)
                print 'Not supported id: %s' % x['ID']

        count = self.current.GetItemCount()

        if count > 0:
            while select > count:
                select = select - 1
            self.current.SetItemState(select, wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED, wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED)
        else:
            if hasattr(self, 'editor'):
                self.sizer.Detach(self.editor)
                self.editor.Destroy()
                del self.editor
            self.editor = wx.StaticText(self, -1, _('Create new message by adding part to left list...'), size=(-1, 150))
            self.sizer.Add(self.editor, pos=(self.editorrow, 1), flag=wx.EXPAND, span=wx.GBSpan(colspan=7))
            self.sizer.Layout()

    def AvailableSelected(self, event):
        self.availsel = event.m_itemIndex

    def OnUnicode(self, event):
        self.entry['SMSInfo']['Unicode'] = self.unicode.GetValue()
        if hasattr(self.editor, 'OnUnicode'):
            self.editor.OnUnicode(self.unicode.GetValue())

    def CurrentSelected(self, event):
        self.StoreEdited()
        if hasattr(self, 'editor'):
            self.sizer.Detach(self.editor)
            self.editor.Destroy()
            del self.editor

        found = False
        for p in SMSParts:
            if self.entry['SMSInfo']['Entries'][event.m_itemIndex]['ID'] in p[2]:
                self.editor = p[3](self, self.entry['SMSInfo']['Entries'][event.m_itemIndex], self.cfg, self.unicode.GetValue())
                self.sizer.Add(self.editor, pos=(self.editorrow, 1), flag=wx.EXPAND, span=wx.GBSpan(colspan=7))
                found = True
                break
        if not found:
            self.editor = wx.StaticText(self, -1, _('No editor available for type %s') % self.entry['SMSInfo']['Entries'][event.m_itemIndex]['ID'])
            self.sizer.Add(self.editor, pos=(self.editorrow, 1), flag=wx.EXPAND, span=wx.GBSpan(colspan=7))
            self.prevedit = -1
        else:
            self.prevedit = event.m_itemIndex
        self.sizer.Layout()

    def UpPressed(self, evt):
        if self.prevedit == -1:
            return
        next = self.prevedit - 1
        if next < 0:
            return
        self.StoreEdited()
        v = self.entry['SMSInfo']['Entries'][self.prevedit]
        self.entry['SMSInfo']['Entries'][self.prevedit] = self.entry['SMSInfo']['Entries'][next]
        self.entry['SMSInfo']['Entries'][next] = v
        self.prevedit = -1
        self.GenerateCurrent(next)

    def DnPressed(self, evt):
        if self.prevedit == -1:
            return
        next = self.prevedit + 1
        if next >= self.current.GetItemCount():
            return
        self.StoreEdited()
        v = self.entry['SMSInfo']['Entries'][self.prevedit]
        self.entry['SMSInfo']['Entries'][self.prevedit] = self.entry['SMSInfo']['Entries'][next]
        self.entry['SMSInfo']['Entries'][next] = v
        self.prevedit = -1
        self.GenerateCurrent(next)

    def DeletePressed(self, evt):
        if self.prevedit == -1:
            return
        self.StoreEdited()
        del self.entry['SMSInfo']['Entries'][self.prevedit]
        next = self.prevedit - 1
        self.prevedit = -1
        self.GenerateCurrent(max(next, 0))

    def AddPressed(self, evt):
        if self.availsel == -1:
            return
        v = {'ID': SMSParts[self.availsel][4]}
        if v['ID'][-5:] == '16bit' and self.cfg.Read('/Message/16bitId') != 'yes':
            v['ID'] = v['ID'][:-5]
        self.StoreEdited()
        self.entry['SMSInfo']['Entries'].insert(self.prevedit + 1, v)
        next = self.prevedit + 1
        self.prevedit = -1
        self.GenerateCurrent(next)

    def StoreEdited(self):
        if self.prevedit != -1:
            self.entry['SMSInfo']['Entries'][self.prevedit] = self.editor.GetValue()

        if self.report.GetValue():
            self.entry['Type'] = 'Status_Report'
        else:
            self.entry['Type'] = 'Submit'

        if self.sent.GetValue():
            self.entry['State'] = 'Sent'
        else:
            self.entry['State'] = 'UnSent'

        if self.flash.GetValue():
            self.entry['SMSInfo']['Class'] = 0
        else:
            self.entry['SMSInfo']['Class'] = 1

        self.entry['Numbers'] = Wammu.PhoneValidator.SplitNumbers(self.number.GetValue())
        self.entry['SMSInfo']['Unicode'] = self.unicode.GetValue()
        self.entry['Save'] = self.save.GetValue()
        self.entry['Send'] = self.send.GetValue()
        self.entry['Folder'] = self.folder.GetValue()

    def Preview(self, evt):
        if len(self.entry['SMSInfo']['Entries']) == 0:
            dlg = wx.MessageDialog(
                self,
                _('Nothing to preview, message is empty.'),
                _('Message empty!'),
                wx.OK | wx.ICON_WARNING
            )
        else:
            self.StoreEdited()
            msg = gammu.EncodeSMS(self.entry['SMSInfo'])
            info = gammu.DecodeSMS(msg)
            result = {}
            result['SMS'] = msg
            if info is not None:
                result['SMSInfo'] = info
            Wammu.Utils.ParseMessage(result, (info is not None))
            dlg = MessagePreview(self, ('<i>%s</i><hr>' % (_('Message will fit into %d SMSes') % len(msg))) + Wammu.MessageDisplay.SmsToHtml(self.cfg, result))

        dlg.ShowModal()
        del dlg

    def Okay(self, evt):
        if not self.number.GetValidator().Validate(self):
            return

        self.StoreEdited()

        self.EndModal(wx.ID_OK)



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
IMAP UTF-7 codec
'''

import codecs

def modified_base64(s):
    s_utf7 = s.encode('utf-7')
    return s_utf7[1:-1].replace('/', ',')

def modified_unbase64(s):
    s_utf7 = '+' + s.replace(',', '/') + '-'
    return s_utf7.decode('utf-7')

def encoder(s):
    r = []
    _in = []
    for c in s:
        if ord(c) in (range(0x20, 0x26) + range(0x27, 0x7f)):
            if _in:
                r.extend(['&', modified_base64(''.join(_in)), '-'])
                del _in[:]
            r.append(str(c))
        elif c == '&':
            if _in:
                r.extend(['&', modified_base64(''.join(_in)), '-'])
                del _in[:]
            r.append('&-')
        else:
            _in.append(c)
    if _in:
        r.extend(['&', modified_base64(''.join(_in)), '-'])
    return (''.join(r), len(s))

def decoder(s):
    r = []
    decode = []
    for c in s:
        if c == '&' and not decode:
            decode.append('&')
        elif c == '-' and decode:
            if len(decode) == 1:
                r.append('&')
            else:
                r.append(modified_unbase64(''.join(decode[1:])))
            decode = []
        elif decode:
            decode.append(c)
        else:
            r.append(c)
    if decode:
        r.append(modified_unbase64(''.join(decode[1:])))
    return (u''.join(r), len(s))

class StreamReader(codecs.StreamReader):
    def decode(self, s, errors='strict'):
        return decoder(s)

class StreamWriter(codecs.StreamWriter):
    def decode(self, s, errors='strict'):
        return encoder(s)

def imap4_utf_7(name):
    if name == 'imap4-utf-7':
        return (encoder, decoder, StreamReader, StreamWriter)

codecs.register(imap4_utf_7)



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:

from unittest import TestCase
from Wammu.Paths import CheckImagesPath, DATAPATH


class PathTest(TestCase):
    def test_path(self):
        self.assertTrue(CheckImagesPath(DATAPATH))



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Logging window and thread for log reading
'''

import threading
import wx
import os
import sys
import time
import Wammu.Events
from Wammu.Locales import ugettext as _


class LoggerDebug(threading.Thread):
    '''
    Thread which reads defined files and prints it to stderr.
    '''
    def __init__(self, filename):
        '''
        Initializes reader on filename, text will be printed to stderr.
        '''
        threading.Thread.__init__(self)
        self.file_descriptor = open(filename, 'r')
        self.filename = filename
        self.canceled = False

    def run(self):
        """
        This is basically tail -f reimplementation
        """
        while not self.canceled:
            where = self.file_descriptor.tell()
            txt = self.file_descriptor.readlines()
            if len(txt) == 0:
                fd_results = os.fstat(self.file_descriptor.fileno())
                try:
                    st_results = os.stat(self.filename)
                except OSError:
                    st_results = fd_results

                if st_results[1] == fd_results[1] or sys.platform == 'win32':
                    time.sleep(1)
                    self.file_descriptor.seek(where)
                else:
                    self.file_descriptor = open(self.filename, 'r')
            else:
                sys.stderr.write(''.join(txt))
        self.file_descriptor.close()

class Logger(threading.Thread):
    '''
    Thread which reads defined files and posts events on change.
    '''
    def __init__(self, win, filename):
        '''
        Initializes reader on filename, events will be sent to win.
        '''
        threading.Thread.__init__(self)
        self.win = win
        self.file_descriptor = open(filename, 'r')
        self.filename = filename
        self.canceled = False

    def run(self):
        """
        This is basically tail -f reimplementation
        """
        while not self.canceled:
            where = self.file_descriptor.tell()
            txt = self.file_descriptor.readlines()
            if len(txt) == 0:
                fd_results = os.fstat(self.file_descriptor.fileno())
                try:
                    st_results = os.stat(self.filename)
                except OSError:
                    st_results = fd_results

                if st_results[1] == fd_results[1] or sys.platform == 'win32':
                    time.sleep(1)
                    self.file_descriptor.seek(where)
                else:
                    self.file_descriptor = open(self.filename, 'r')
            else:
                evt = Wammu.Events.LogEvent(txt=''.join(txt))
                wx.PostEvent(self.win, evt)
        self.file_descriptor.close()

class LogFrame(wx.Frame):
    '''
    Window with debug log.
    '''

    def __init__(self, parent, cfg):
        '''
        Creates window and initializes event handlers.
        '''
        self.cfg = cfg
        if cfg.HasEntry('/Debug/X') and cfg.HasEntry('/Debug/Y'):
            pos = wx.Point(
                    cfg.ReadInt('/Debug/X'),
                    cfg.ReadInt('/Debug/Y'))
        else:
            pos = wx.DefaultPosition
        size = wx.Size(
            cfg.ReadInt('/Debug/Width'),
            cfg.ReadInt('/Debug/Height')
        )
        wx.Frame.__init__(
            self,
            parent,
            -1,
            _('Wammu debug log'),
            pos,
            size,
            wx.DEFAULT_FRAME_STYLE | wx.RESIZE_BORDER
        )
        self.txt = wx.TextCtrl(
            self,
            -1,
            _('Here will appear debug messages from Gammu...\n'),
            style=wx.TE_MULTILINE | wx.TE_READONLY
        )
        self.txt.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
        Wammu.Events.EVT_LOG(self, self.OnLog)
        wx.EVT_SIZE(self, self.OnSize)
        self.OnSize(None)

    def OnLog(self, evt):
        '''
        Event handler for text events from Logger.
        '''
        self.txt.AppendText(evt.txt)

    def OnSize(self, evt):
        '''
        Resize handler to correctly resize text area.
        '''
        width, height = self.GetClientSizeTuple()
        self.txt.SetDimensions(0, 0, width, height)



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Settings storage and configuration manager
'''

import sys
import Wammu.Paths
import Wammu.Data
import Wammu.Utils
from Wammu.Locales import ugettext as _

COM_PORTS = 16
UNX_DEVICES = 4


class Settings:
    """
    Helper class for generating settings.
    """
    def __init__(self):
        self.connection = None
        self.driver = None
        self.manufacturer = None
        self.name = None
        self.port = None
        self.gammudriver = None
        self.position = 0

    def GetName(self):
        if self.name is None:
            if self.position == 0:
                return 'gammu'
            else:
                return 'gammu%d' % self.position
        else:
            return self.name

    def GetGammuDriver(self):
        return self.gammudriver

    def GetPort(self):
        return self.port

    def GetConnection(self):
        return self.connection

    def SetPosition(self, pos):
        self.position = pos

    def SetConnection(self, conn):
        self.connection = conn

    def SetDriver(self, driver):
        self.driver = driver

    def SetGammuDriver(self, driver):
        self.gammudriver = driver

    def SetManufacturer(self, manu):
        self.manufacturer = manu

    def SetPort(self, port):
        self.port = port

    def SetName(self, name):
        self.name = name

    def GetSettings(self):
        return {'Position': self.position, 'Device': self.port, 'Connection': self.gammudriver, 'Name': self.name}

    def GetManufacturers(self):
        names = []
        connections = []
        helps = []

        names.append('any')
        connections.append(_('I don\'t know'))
        helps.append(_('Select this option only if really necessary. You will be provided with too much options in next step.'))

        names.append('symbian')
        connections.append(_('Symbian based phone'))
        helps.append(_('Go on if your phone uses Symbian OS (regardless of manufacturer).'))

        names.append('nota')
        connections.append(_('Alcatel phone'))
        helps.append(_('Alcatel phone not running Symbian.'))

        names.append('nota')
        connections.append(_('BenQ/Siemens phone'))
        helps.append(_('BenQ or Siemens phone not running Symbian.'))

        names.append('nota')
        connections.append(_('Motorola phone'))
        helps.append(_('Motorola phone not running Symbian.'))

        names.append('nokia')
        connections.append(_('Nokia phone'))
        helps.append(_('Nokia phone not running Symbian.'))

        names.append('nota')
        connections.append(_('Samsung phone'))
        helps.append(_('Samsung phone not running Symbian.'))

        names.append('nota')
        connections.append(_('Sharp phone'))
        helps.append(_('Sharp phone not running Symbian.'))

        names.append('nota')
        connections.append(_('Sony Ericsson phone'))
        helps.append(_('Sony Ericsson phone not running Symbian.'))

        names.append('nota')
        connections.append(_('None of the above'))
        helps.append(_('Select this option if nothing above matches.'))

        return (names, connections, helps)

    def AddOBEX(self, names, connections, helps):
        names.append('obex')
        connections.append(_('OBEX and IrMC protocols'))
        if self.manufacturer in ['symbian', 'nokia']:
            helps.append(_('Standard access to filesystem. Not a good choice for Nokia if you want to access data.'))
        else:
            helps.append(_('Standard access to filesystem and sometimes also to phone data. Good choice for recent phones.'))

    def AddSymbian(self, names, connections, helps):
        names.append('symbian')
        connections.append(_('Symbian using Gnapplet'))
        helps.append(_('You have to install Gnapplet into phone before using this connection. You can find it in Gammu sources.'))

    def AddNokia(self, names, connections, helps):
        names.append('fbus')
        connections.append(_('Nokia proprietary protocol'))
        helps.append(_('Nokia proprietary protocol FBUS.'))
        if self.connection == 'serial':
            names.append('mbus')
            connections.append(_('Nokia proprietary service protocol'))
            helps.append(_('Nokia proprietary protocol MBUS. Older version, use FBUS if possible.'))

    def AddAT(self, names, connections, helps):
        names.append('at')
        connections.append(_('AT based'))
        if self.manufacturer in ['symbian', 'nokia']:
            helps.append(_('This provides minimal access to phone features. It is recommended to use other connection type.'))
        else:
            helps.append(_('Good choice for most phones except Nokia and Symbian based. Provides access to most phone features.'))

    def GetDrivers(self):
        names = []
        connections = []
        helps = []

        if self.manufacturer == 'nokia':
            self.AddNokia(names, connections, helps)
            self.AddAT(names, connections, helps)
            self.AddOBEX(names, connections, helps)
        elif self.manufacturer == 'symbian':
            self.AddSymbian(names, connections, helps)
            self.AddAT(names, connections, helps)
            self.AddOBEX(names, connections, helps)
        elif self.manufacturer == 'nota':
            self.AddAT(names, connections, helps)
            self.AddOBEX(names, connections, helps)
        elif self.manufacturer == 'any':
            self.AddAT(names, connections, helps)
            self.AddOBEX(names, connections, helps)
            self.AddNokia(names, connections, helps)
            self.AddSymbian(names, connections, helps)

        return (names, connections, helps)

    def GetPortType(self):
        if self.gammudriver in [
                'mbus',
                'fbus',
                'dlr3',
                'at',
                'at19200',
                'at38400',
                'at115200',
                'obex',
                'phonetblue',
                'fbusblue',
                'fbus-nodtr',
                'dku5-nodtr']:
            if self.connection == 'serial':
                return 'serial'
            elif self.connection == 'bluetooth':
                return 'btserial'
            elif self.connection == 'irda':
                return 'irdaserial'
            elif self.connection == 'usb':
                return 'usbserial'
            return 'serial'
        if self.gammudriver in [
                'blueat',
                'bluerfat',
                'blueobex',
                'bluerfobex',
                'bluerfgnapbus',
                'bluerffbus',
                'bluephonet',
                'bluerfphonet']:
            return 'bluetooth'
        if self.gammudriver in [
                'dku2',
                'dku5',
                'dku2at']:
            return 'dku'
        if self.gammudriver in [
                'irdaat',
                'irdaobex',
                'irdagnapbus',
                'fbusirda',
                'irdaphonet']:
            return 'irda'
        if self.gammudriver is None:
            return None
        # fallback
        return None

    def GetBluezDevices(self):
        try:
            import bluetooth
            return bluetooth.discover_devices()
        except ImportError:
            return []
        except bluetooth.BluetoothError:
            return []
        except IOError:
            return []

    def CheckDev(self, dev):
        res = Wammu.Utils.CheckDeviceNode(dev)
        if res[0] == 0:
            return True
        else:
            return False

    def AddDevs(self, lst, format, limit):
        for x in range(limit):
            name = format % x
            if self.CheckDev(name):
                lst.append(name)


    def GetDevicesWindows(self):
        type = self.GetPortType()
        result = []
        if type in ['serial', 'btserial', 'irdaserial', 'usbserial', None]:
            self.AddDevs(result, 'COM%d', COM_PORTS)
        if type in ['bluetooth', None]:
            result += self.GetBluezDevices()

        help = ''
        if type == 'serial':
            help = _('Enter device name of serial port.')
        elif type in ['btserial', 'irdaserial', 'usbserial']:
            help = _('Enter device name of emulated serial port.')
        elif type == 'bluetooth':
            help = _('Enter Bluetooth address of your phone.')
        elif type in ['irda', 'dku']:
            help = _('You don\'t have to enter anything for this settings.')

        return result, help

    def GetDevicesUNIX(self):
        type = self.GetPortType()
        result = []
        if type in ['serial', None]:
            self.AddDevs(result, '/dev/cua%d', UNX_DEVICES)
            self.AddDevs(result, '/dev/ttyS%d', UNX_DEVICES)
            self.AddDevs(result, '/dev/tts/%d', UNX_DEVICES)
        if type in ['btserial', None]:
            self.AddDevs(result, '/dev/rfcomm%d', UNX_DEVICES)
        if type in ['irdaserial', None]:
            self.AddDevs(result, '/dev/ircomm%d', UNX_DEVICES)
        if type in ['usbserial', 'dku', None]:
            self.AddDevs(result, '/dev/ttyACM%d', UNX_DEVICES)
            self.AddDevs(result, '/dev/ttyUSB%d', UNX_DEVICES)
            self.AddDevs(result, '/dev/usb/tts/%d', UNX_DEVICES)
            self.AddDevs(result, '/dev/usb/acm/%d', UNX_DEVICES)
            self.AddDevs(result, '/dev/input/ttyACM%d', UNX_DEVICES)
        if type in ['bluetooth', None]:
            result += self.GetBluezDevices()

        help = ''
        if type == 'serial':
            help = _('Enter device name of serial port.')
        elif type in ['btserial', 'irdaserial']:
            help = _('Enter device name of emulated serial port.')
        elif type == 'bluetooth':
            help = _('Enter Bluetooth address of your phone.')
        elif type in ['usbserial', 'dku']:
            help = _('Enter device name of USB port.')
        elif type in ['irda', 'dku']:
            help = _('You don\'t have to enter anything for this settings.')

        return result, help


    def GetDevices(self):
        if sys.platform == 'win32':
            return self.GetDevicesWindows()
        else:
            return self.GetDevicesUNIX()

    def GetGammuDrivers(self):
        names = []
        connections = []
        helps = []

        if self.driver == 'at':
            if self.connection != 'bluetooth':
                names.append('at')
                connections.append(_('Generic AT over serial line or it\'s emulation'))
                helps.append(_('Select this if you have real serial port or it is emulated by phone driver (eg. virtual COM port, /dev/rfcomm, /dev/ircomm, etc.).'))

            if self.connection == 'serial':
                for rate in [19200, 38400, 115200]:
                    names.append('at%d' % rate)
                    connections.append(_('Generic AT at %d bps') % rate)
                    helps.append(_('Select this if your phone requires transfer speed %d bps.') % rate)

            elif self.connection == 'bluetooth':
                names.append('blueat')
                connections.append(_('AT over Bluetooth'))
                helps.append(_('Select this if your phone is connected over Bluetooth and you want to use native Bluetooth connection.'))

                names.append('at')
                connections.append(_('Generic AT over serial line or it\'s emulation'))
                helps.append(_('Select this if you have real serial port or it is emulated by phone driver (eg. virtual COM port, /dev/rfcomm, /dev/ircomm, etc.).'))

                names.append('bluerfat')
                connections.append(_('AT over Bluetooth with RF searching'))
                helps.append(_('Use for Bluetooth stack and 6210 / DCT4 Nokia models, which don\'t inform about Bluetooth services correctly (6310, 6310i with firmware lower than 5.50, 8910,..)'))

            elif self.connection == 'irda':
                names.append('irdaat')
                connections.append(_('AT over IrDA'))
                helps.append(_('Select this if your phone is connected over IrDA and you want to use native IrDA connection.'))

            elif self.manufacturer == 'nokia':
                names.append('dku2at')
                connections.append(_('AT over DKU2'))
                helps.append(_('Select this if your phone is connected using DKU2 cable.'))

        elif self.driver == 'obex':
            names.append('obex')
            connections.append(_('Generic OBEX over serial line or it\'s emulation'))
            helps.append(_('Select this if you have real serial port or it is emulated by phone driver (eg. virtual COM port, /dev/rfcomm, /dev/ircomm, etc.).'))

            if self.connection == 'bluetooth':
                names.append('blueobex')
                connections.append(_('OBEX over Bluetooth'))
                helps.append(_('Select this if your phone is connected over Bluetooth and you want to use native Bluetooth connection.'))

                names.append('bluerfobex')
                connections.append(_('OBEX over Bluetooth with RF searching'))
                helps.append(_('Use for Bluetooth stack and 6210 / DCT4 Nokia models, which don\'t inform about Bluetooth services correctly (6310, 6310i with firmware lower than 5.50, 8910,..)'))

            elif self.connection == 'irda':
                names.append('irdaobex')
                connections.append(_('OBEX over IrDA'))
                helps.append(_('Select this if your phone is connected over IrDA and you want to use native IrDA connection.'))

        elif self.driver == 'symbian':
            if self.connection == 'bluetooth':
                names.append('bluerfgnapbus')
                connections.append(_('Gnapplet over Bluetooth'))
                helps.append(_('Select this if your phone is connected over Bluetooth and you want to use native Bluetooth connection.'))

            elif self.connection == 'irda':
                names.append('irdagnapbus')
                connections.append(_('Gnapplet over IrDA'))
                helps.append(_('Select this if your phone is connected over IrDA and you want to use native IrDA connection.'))

        elif self.driver == 'mbus':
            if self.connection == 'serial':
                names.append('mbus')
                connections.append(_('MBUS proprietary protocol'))
                helps.append(_('Protocol used in older Nokia phones.'))

        elif self.driver == 'fbus':
            if self.connection == 'serial':
                names.append('fbus')
                connections.append(_('FBUS proprietary protocol'))
                helps.append(_('Protocol used in Nokia phones. Please try selecting more specific options first.'))

            # Serial should not be here, but we do not trust people they really have serial :-)
            if self.connection in ['serial', 'usb']:
                names.append('dku5')
                connections.append(_('DKU5 cable'))
                helps.append(_('Nokia Connectivity Adapter Cable DKU-5 (original cable or compatible), for phones with USB chip like Nokia 5100.'))

                names.append('fbuspl2303')
                connections.append(_('PL2303 cable'))
                helps.append(_('New Nokia protocol for PL2303 USB cable (usually third party cables), for phones with USB chip like Nokia 5100.'))

                names.append('dku2')
                connections.append(_('DKU2 cable'))
                helps.append(_('Nokia Connectivity Cable DKU-2 (original cable or compatible), for phones without USB chip like Nokia 6230.'))

                names.append('dlr3')
                connections.append(_('DLR3-3P/CA-42 cable'))
                helps.append(_('Nokia RS-232 Adapter Cable DLR-3P (original cable or compatible), usually with phones like Nokia 7110/6210/6310.'))

                names.append('fbus-nodtr')
                connections.append(_('FBUS proprietary protocol using ARK cable'))
                helps.append(_('ARK cable (third party cable) for phones not supporting AT commands like Nokia 6020.'))

                names.append('dku5-nodtr')
                connections.append(_('DKU5 phone with ARK cable'))
                helps.append(_('ARK cable (third party cable) for phones with USB chip like Nokia 5100.'))

            elif self.connection == 'bluetooth':
                names.append('bluephonet')
                connections.append(_('Phonet over Bluetooth'))
                helps.append(_('Nokia protocol for Bluetooth stack with other DCT4 Nokia models.'))

                names.append('fbusblue')
                connections.append(_('FBUS over Bluetooth (emulated serial port)'))
                helps.append(
                    _('Nokia protocol for Bluetooth stack with Nokia 6210.') +
                    ' ' +
                    _('Using emulated serial port.')
                )

                names.append('phonetblue')
                connections.append(_('Phonet over Bluetooth (emulated serial port)'))
                helps.append(
                    _('Nokia protocol for Bluetooth stack with other DCT4 Nokia models.') +
                    ' ' +
                    _('Using emulated serial port.')
                )

                names.append('bluerffbus')
                connections.append(_('FBUS over Bluetooth'))
                helps.append(_('Nokia protocol for Bluetooth stack with Nokia 6210.'))

                names.append('bluerfphonet')
                connections.append(_('Phonet over Bluetooth with RF searching'))
                helps.append(_('Nokia protocol for Bluetooth stack with DCT4 Nokia models, which don\'t inform about services correctly (6310, 6310i with firmware lower than 5.50, 8910,..).'))

            elif self.connection == 'irda':
                names.append('irdaphonet')
                connections.append(_('Phonet over IrDA'))
                helps.append(_('Nokia protocol for infrared with other Nokia models.'))

                names.append('fbusirda')
                connections.append(_('FBUS over IrDA'))
                helps.append(_('Nokia protocol for infrared with Nokia 6110/6130/6150.'))

        return (names, connections, helps)



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
SMS export code.
'''

import os
import imaplib
import wx
import Wammu.MailWriter
import Wammu.IMAP
import re
from Wammu.Locales import ugettext as _


MAILBOX_RE = re.compile(r'\((?P<flags>(\s*\\\w*)*)\)\s+(?P<delim>[^ ]*)\s+(?P<name>.*)')
POSSIBLY_QUOTED_RE = re.compile(r'"?([^"]*)"?')

def SMSToMailbox(parent, messages, contacts):
    count = len(messages)
    wildcard = _('Mailboxes') + ' (*.mbox)|*.mbox|' + _('All files') + ' (*.*)|*.*;*'
    exts = ['mbox']
    exts.append(None)
    dlg = wx.FileDialog(parent, _('Select mailbox file...'), os.getcwd(), "", wildcard, wx.SAVE | wx.OVERWRITE_PROMPT | wx.CHANGE_DIR)

    if dlg.ShowModal() != wx.ID_OK:
        return

    path = dlg.GetPath()
    ext = exts[dlg.GetFilterIndex()]
    # Add automatic extension if we know one and file does not
    # have any
    if (os.path.splitext(path)[1] == '' and
            ext is not None):
        path += '.' + ext

    parent.ShowProgress(_('Saving messages to mailbox'))
    try:
        f = file(path, 'w')
        for i in range(count):
            if not parent.progress.Update(i * 100 / count):
                del parent.progress
                parent.SetStatusText(_('Export terminated'))
                return

            sms = messages[i]
            filename, data, unused_msgid = Wammu.MailWriter.SMSToMail(parent.cfg, sms, contacts, True)
            f.write(data)
            f.write('\n')
            f.write('\n')

        f.close()
    except IOError:
        wx.MessageDialog(
            parent,
            _('Creating of file %s failed, bailing out.') % path,
            _('Can not create file!'),
            wx.OK | wx.ICON_ERROR
        ).ShowModal()
        del parent.progress
        parent.SetStatusText(_('Export terminated'))
        return

    parent.progress.Update(100)
    del parent.progress
    parent.SetStatusText(
        _('%(count)d messages exported to "%(path)s" (%(type)s)') % {
            'count': count,
            'path': path,
            'type': _('mailbox')
        }
    )

def SMSToMaildir(parent, messages, contacts):
    count = len(messages)
    dlg = wx.DirDialog(
        parent,
        _('Select maildir directory where to save files'),
        os.getcwd(),
        style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON
    )

    if dlg.ShowModal() != wx.ID_OK:
        return

    path = dlg.GetPath()
    outpath = path

    if not os.path.isdir(os.path.join(outpath, 'new')):
        res = wx.MessageDialog(
            parent,
            _('Selected folder does not contain new subfolder and thus probably is not valid maildir.\n\nDo you want to create new subfolder and export to it?'),
            _('Folder doesn\'t look like maildir!'),
            wx.YES_NO | wx.YES_DEFAULT | wx.CANCEL | wx.ICON_WARNING
        ).ShowModal()

        if res == wx.ID_CANCEL:
            return

        if res == wx.ID_YES:
            outpath = os.path.join(outpath, 'new')
            try:
                os.mkdir(outpath)
            except OSError:
                wx.MessageDialog(
                    parent,
                    _('Creating of folder failed, bailing out.'),
                    _('Can not create folder!'),
                    wx.OK | wx.ICON_ERROR
                ).ShowModal()
                return
    else:
        outpath = os.path.join(outpath, 'new')

    parent.ShowProgress(_('Saving messages to maildir'))
    for i in range(count):
        if not parent.progress.Update(i * 100 / count):
            del parent.progress
            parent.SetStatusText(_('Export terminated'))
            return

        sms = messages[i]
        filename, data, unused_msgid = Wammu.MailWriter.SMSToMail(parent.cfg, sms, contacts)

        outfile = os.path.join(outpath, filename)
        if os.path.exists(outfile):
            res = wx.MessageDialog(
                parent,
                _('Output file already exists, this usually means that this message was already saved there.\n\nDo you wish to overwrite file %s?') % outfile,
                _('File already exists!'),
                wx.YES_NO | wx.NO_DEFAULT | wx.CANCEL | wx.ICON_WARNING
            ).ShowModal()

            if res == wx.ID_CANCEL:
                del parent.progress
                parent.SetStatusText(_('Export terminated'))
                return

            if res == wx.ID_NO:
                continue
        try:
            f = file(outfile, 'w')
            f.write(data)
            f.close()
        except IOError:
            wx.MessageDialog(
                parent,
                _('Creating of file %s failed, bailing out.') % outfile,
                _('Can not create file!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            del parent.progress
            parent.SetStatusText(_('Export terminated'))
            return

    parent.progress.Update(100)
    del parent.progress

    parent.SetStatusText(
        _('%(count)d messages exported to "%(path)s" (%(type)s)') % {
            'count': count,
            'path': path,
            'type': _('maildir')
        }
    )

def ParseIMAPFolder(item):
    '''
    Parses folder reply from IMAP.
    '''
    match = MAILBOX_RE.match(item)

    if not match:
        return (None, None)

    delim = POSSIBLY_QUOTED_RE.match(match.group('delim')).group(1)
    path = POSSIBLY_QUOTED_RE.match(match.group('name')).group(1)
    flags = match.group('flags')
    return (path, flags)

def SMSToIMAP(parent, messages, contacts):
    imapConfig = IMAPConfigHelper(parent.cfg)

    while True:
        value = IMAPSettingsDialog(parent, imapConfig).ShowModal()
        if value == wx.ID_CANCEL:
             return

        busy = wx.BusyInfo(_('Connecting to IMAP server...'))

        if imapConfig.useSSL:
            m = imaplib.IMAP4_SSL(imapConfig.server, int(imapConfig.port))
        else:
            m = imaplib.IMAP4(imapConfig.server, int(imapConfig.port))

        try:
            res = m.login(imapConfig.login, imapConfig.password)
        except:
            res = ['FAIL']
        del busy
        if res[0] == 'OK':
            break
        else:
            dialog = wx.MessageDialog(
                parent,
                _('Can not login, you probably entered invalid login information. Do you want to retry?'),
                _('Login failed!'),
                wx.YES_NO | wx.YES_DEFAULT | wx.ICON_ERROR
            )
            if dialog.ShowModal() == wx.ID_NO:
                return

    busy = wx.BusyInfo(_('Listing folders on IMAP server...'))
    try:
        res, list = m.list()
    except:
        res = 'FAIL'
    del busy
    if res != 'OK':
        wx.MessageDialog(
            parent,
            _('Can not list folders on server, bailing out.'),
            _('Listing failed!'),
            wx.OK | wx.ICON_ERROR
        ).ShowModal()
        parent.SetStatusText(_('Export terminated'))
        return

    folders = []
    for item in list:
        path, flags = ParseIMAPFolder(item)

        if path is None:
            continue

        if flags.find('\\Noselect') != -1:
            continue

        try:
            folders.append(unicode(path, 'imap4-utf-7'))
        except UnicodeDecodeError:
            # Ignore folders which can not be properly converted
            pass

    folders.sort()

    lastFolder = parent.cfg.Read('/IMAP/LastUsedFolder')
    folderIndex = 0
    if lastFolder != '':
        try:
            folderIndex = folders.index(lastFolder)
        except:
            pass

    dlg = wx.SingleChoiceDialog(
        parent,
        _('Please select folder on server %s where messages will be stored') % imapConfig.server,
        _('Select folder'),
        folders, wx.CHOICEDLG_STYLE | wx.RESIZE_BORDER
    )
    if folderIndex > 0:
        dlg.SetSelection(folderIndex)
    if dlg.ShowModal() == wx.ID_CANCEL:
        try:
            m.logout()
        except:
            pass
        return
    path = '%s@%s/%s' % (imapConfig.login, imapConfig.server, folders[dlg.GetSelection()])
    folder = folders[dlg.GetSelection()].encode('imap4-utf-7')

    parent.cfg.Write('/IMAP/LastUsedFolder', folders[dlg.GetSelection()])

    busy = wx.BusyInfo(_('Selecting folder on IMAP server...'))
    try:
        res = m.select(folder)
    except:
        res = ['FAIL']
    del busy
    if res[0] != 'OK':
        wx.MessageDialog(
            parent,
            _('Can not select folder %s on server, bailing out.') % folder,
            _('Selecting failed!'),
            wx.OK | wx.ICON_ERROR
        ).ShowModal()
        parent.SetStatusText(_('Export terminated'))
        return

    def msgFilter(sms):
        return (
            (imapConfig.backupRead and sms['SMS'][0]['State'] == 'Read') or
            (imapConfig.backupSent and sms['SMS'][0]['State'] == 'Sent') or
            (imapConfig.backupUnread and sms['SMS'][0]['State'] == 'UnRead')or
            (imapConfig.backupUnsent and sms['SMS'][0]['State'] == 'UnSent')
        )

    messages = filter(msgFilter, messages)
    count = len(messages)

    parent.ShowProgress(_('Saving messages to IMAP'))

    new_messages_num = 0
    count = len(messages)
    for i in range(count):
        if not parent.progress.Update(i * 100 / count):
            del parent.progress
            parent.SetStatusText(_('Export terminated'))
            return

        sms = messages[i]

        filename, data, msgid = Wammu.MailWriter.SMSToMail(parent.cfg, sms, contacts)

        if imapConfig.newMessages:
            res, msgnums = m.search(None, 'HEADER', '"Message-ID" "' + msgid + '"')
            if len(msgnums[0].split()) != 0:
                continue

        new_messages_num += 1

        try:
            res = m.append(folder, '$SMS', None, data)
        except:
            res = ['FAIL']
        if res[0] != 'OK':
            wx.MessageDialog(
                parent,
                _('Can not save message to folder %s on server, bailing out.') % folder,
                _('Saving failed!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            parent.progress.Update(100)
            del parent.progress
            parent.SetStatusText(_('Export terminated'))
            return

    parent.progress.Update(100)
    del parent.progress

    try:
        m.logout()
    except:
        pass

    if not imapConfig.newMessages:
        parent.SetStatusText(
            _('%(count)d messages exported to "%(path)s" (%(type)s)') % {
                'count': count,
                'path': path,
                'type': _('IMAP server')
            }
        )
    else:
        parent.SetStatusText(
            _('%(new)d new of %(count)d messages exported to "%(path)s" (%(type)s)') % {
                'new': new_messages_num,
                'count': count,
                'path': path,
                'type': _('IMAP server')
            }
        )

def SMSExport(parent, messages, contacts):
    # Select where to export
    dlg = wx.SingleChoiceDialog(
        parent,
        _('Where do you want to export emails created from your messages?'),
        _('Select export type'),
        [_('Mailbox file'), _('Maildir folder'), _('IMAP account')],
        wx.CHOICEDLG_STYLE | wx.RESIZE_BORDER
    )
    if dlg.ShowModal() != wx.ID_OK:
        return

    idx = dlg.GetSelection()
    del dlg

    # Mailbox
    if idx == 0:
        SMSToMailbox(parent, messages, contacts)
    # Maildir
    elif idx == 1:
        SMSToMaildir(parent, messages, contacts)
    # IMAP
    elif idx == 2:
        SMSToIMAP(parent, messages, contacts)
    else:
        raise Exception('Not implemented export functionality!')

def bool2yn(value):
    if value:
        return 'yes'
    else:
        return 'no'

def yn2bool(value):
    return value == 'yes'

class IMAPConfigHelper:
    '''
    A small helper to read and write the Wammu config
    '''
    def __init__(self, WammuConfig):
        self.wcfg = WammuConfig
        self.load()

    def load(self):

        # Textfields
        self.fromAddress = self.wcfg.Read('/MessageExport/From')
        self.server = self.wcfg.Read('/IMAP/Server')
        self.port = self.wcfg.Read('/IMAP/Port')
        self.login = self.wcfg.Read('/IMAP/Login')
        self.password = self.wcfg.Read('/IMAP/Password')

        # Checkboxes
        self.rememberPassword = yn2bool(self.wcfg.Read('/IMAP/RememberPassword'))
        self.useSSL = yn2bool(self.wcfg.Read('/IMAP/UseSSL'))
        self.newMessages = yn2bool(self.wcfg.Read('/IMAP/OnlyNewMessages'))

        # States
        self.backupRead = yn2bool(self.wcfg.Read('/IMAP/BackupStateRead'))
        self.backupSent = yn2bool(self.wcfg.Read('/IMAP/BackupStateSent'))
        self.backupUnread = yn2bool(self.wcfg.Read('/IMAP/BackupStateUnread'))
        self.backupUnsent = yn2bool(self.wcfg.Read('/IMAP/BackupStateUnsent'))

        if self.port == '':
            if self.useSSL:
                self.port = '993'
            else:
                self.port = '143'

    def write(self):

        # Textfields
        self.wcfg.Write('/MessageExport/From', self.fromAddress)
        self.wcfg.Write('/IMAP/Server', self.server)
        self.wcfg.Write('/IMAP/Port', self.port)
        self.wcfg.Write('/IMAP/Login', self.login)
        if self.rememberPassword:
            self.wcfg.Write('/IMAP/Password', self.password)
        else:
            self.wcfg.Write('/IMAP/Password', '')

        # Checkboxes
        self.wcfg.Write('/IMAP/RememberPassword', bool2yn(self.rememberPassword))
        self.wcfg.Write('/IMAP/UseSSL', bool2yn(self.useSSL))
        self.wcfg.Write('/IMAP/OnlyNewMessages', bool2yn(self.newMessages))

        # States
        self.wcfg.Write('/IMAP/BackupStateRead', bool2yn(self.backupRead))
        self.wcfg.Write('/IMAP/BackupStateSent', bool2yn(self.backupSent))
        self.wcfg.Write('/IMAP/BackupStateUnread', bool2yn(self.backupUnread))
        self.wcfg.Write('/IMAP/BackupStateUnsent', bool2yn(self.backupUnsent))


class IMAPSettingsDialog(wx.Dialog):
    def __init__(self, parent, imapConfig):
        wx.Dialog.__init__(self, parent, -1, _("IMAP Settings"), style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.cfg = imapConfig
        self.connectionFrameSizer_staticbox = wx.StaticBox(self, -1, _("Connection Details"))
        self.preferenciesFrameSizer_staticbox = wx.StaticBox(self, -1, _("Preferences"))
        self.stateFrameSizer_staticbox = wx.StaticBox(self, -1, _("Message State Selection"))
        self.fromAddressLabel = wx.StaticText(self, -1, _("From Address"))
        self.fromAddressTextCtrl = wx.TextCtrl(self, -1, "")
        self.serverLabel = wx.StaticText(self, -1, _("Server"))
        self.serverTextCtrl = wx.TextCtrl(self, -1, "")
        self.portLabel = wx.StaticText(self, -1, _("Port"))
        self.portTextCtrl = wx.TextCtrl(self, -1, "")
        self.loginLabel = wx.StaticText(self, -1, _("Login"))
        self.loginTextCtrl = wx.TextCtrl(self, -1, "")
        self.passwordLabel = wx.StaticText(self, -1, _("Password"))
        self.passwordTextCtrl = wx.TextCtrl(self, -1, "", style=wx.TE_PASSWORD)
        self.rememberCheckBox = wx.CheckBox(self, -1, _("Remember password (insecure)"))
        self.useSSLCheckBox = wx.CheckBox(self, -1, _("Use SSL"))
        self.newMessagesCheckBox = wx.CheckBox(self, -1, _("Only back-up new messages"))
        self.readCheckBox = wx.CheckBox(self, -1, _("Read"))
        self.sentCheckBox = wx.CheckBox(self, -1, _("Sent"))
        self.unreadCheckBox = wx.CheckBox(self, -1, _("Unread"))
        self.unsentCheckBox = wx.CheckBox(self, -1, _("Unsent"))
        self.applyButton = wx.Button(self, wx.ID_APPLY, "")
        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "")
        self.okButton = wx.Button(self, wx.ID_OK, "")

        self.__read_config()
        self.__do_layout()

        self.Bind(wx.EVT_CHECKBOX, self.OnToggleSSL, self.useSSLCheckBox)
        self.Bind(wx.EVT_BUTTON, self.OnOkClick, self.okButton)
        self.Bind(wx.EVT_BUTTON, self.OnApplyClick, self.applyButton)

    def __read_config(self):
        self.fromAddressTextCtrl.SetValue(self.cfg.fromAddress)
        self.serverTextCtrl.SetValue(self.cfg.server)
        self.portTextCtrl.SetValue(self.cfg.port)
        self.loginTextCtrl.SetValue(self.cfg.login)
        self.passwordTextCtrl.SetValue(self.cfg.password)

        self.rememberCheckBox.SetValue(self.cfg.rememberPassword)
        self.useSSLCheckBox.SetValue(self.cfg.useSSL)
        self.newMessagesCheckBox.SetValue(self.cfg.newMessages)

        self.readCheckBox.SetValue(self.cfg.backupRead)
        self.sentCheckBox.SetValue(self.cfg.backupSent)
        self.unreadCheckBox.SetValue(self.cfg.backupUnread)
        self.unsentCheckBox.SetValue(self.cfg.backupUnsent)

    def __copy_config(self):
        self.cfg.fromAddress = self.fromAddressTextCtrl.GetValue()
        self.cfg.server = self.serverTextCtrl.GetValue()
        self.cfg.port = self.portTextCtrl.GetValue()
        self.cfg.login = self.loginTextCtrl.GetValue()
        self.cfg.password = self.passwordTextCtrl.GetValue()

        self.cfg.rememberPassword = self.rememberCheckBox.GetValue()
        self.cfg.useSSL = self.useSSLCheckBox.GetValue()
        self.cfg.newMessages = self.useSSLCheckBox.GetValue()

        self.cfg.backupRead = self.readCheckBox.GetValue()
        self.cfg.backupSent = self.sentCheckBox.GetValue()
        self.cfg.backupUnread = self.unreadCheckBox.GetValue()
        self.cfg.backupUnsent = self.unsentCheckBox.GetValue()

    def __do_layout(self):
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        buttonRowSizer = wx.BoxSizer(wx.HORIZONTAL)
        stateFrameSizer = wx.StaticBoxSizer(self.stateFrameSizer_staticbox, wx.HORIZONTAL)
        preferenciesFrameSizer = wx.StaticBoxSizer(self.preferenciesFrameSizer_staticbox, wx.HORIZONTAL)
        preferenciesGridSizer = wx.FlexGridSizer(3, 2, 2, 2)
        connectionFrameSizer = wx.StaticBoxSizer(self.connectionFrameSizer_staticbox, wx.HORIZONTAL)
        connectionGridSizer = wx.FlexGridSizer(5, 2, 2, 10)
        connectionGridSizer.Add(self.fromAddressLabel, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        connectionGridSizer.Add(self.fromAddressTextCtrl, 0, wx.EXPAND, 0)
        connectionGridSizer.Add(self.serverLabel, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        connectionGridSizer.Add(self.serverTextCtrl, 0, wx.EXPAND, 0)
        connectionGridSizer.Add(self.portLabel, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        connectionGridSizer.Add(self.portTextCtrl, 0, wx.EXPAND, 0)
        connectionGridSizer.Add(self.loginLabel, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        connectionGridSizer.Add(self.loginTextCtrl, 0, wx.EXPAND, 0)
        connectionGridSizer.Add(self.passwordLabel, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        connectionGridSizer.Add(self.passwordTextCtrl, 0, wx.EXPAND, 0)
        connectionGridSizer.AddGrowableCol(1)
        connectionFrameSizer.Add(connectionGridSizer, 1, wx.EXPAND, 0)
        mainSizer.Add(connectionFrameSizer, 0, wx.ALL | wx.EXPAND, 2)
        preferenciesGridSizer.Add((90, 20), 0, 0, 0)
        preferenciesGridSizer.Add(self.rememberCheckBox, 0, 0, 0)
        preferenciesGridSizer.Add((20, 20), 0, 0, 0)
        preferenciesGridSizer.Add(self.useSSLCheckBox, 0, 0, 0)
        preferenciesGridSizer.Add((20, 20), 0, 0, 0)
        preferenciesGridSizer.Add(self.newMessagesCheckBox, 0, 0, 0)
        preferenciesGridSizer.AddGrowableCol(1)
        preferenciesFrameSizer.Add(preferenciesGridSizer, 1, wx.EXPAND, 0)
        mainSizer.Add(preferenciesFrameSizer, 0, wx.ALL | wx.EXPAND, 2)
        stateFrameSizer.Add(self.readCheckBox, 1, wx.EXPAND, 0)
        stateFrameSizer.Add(self.sentCheckBox, 1, wx.EXPAND, 0)
        stateFrameSizer.Add(self.unreadCheckBox, 1, wx.EXPAND, 0)
        stateFrameSizer.Add(self.unsentCheckBox, 1, wx.EXPAND, 0)
        mainSizer.Add(stateFrameSizer, 0, wx.ALL | wx.EXPAND, 2)
        buttonRowSizer.Add((20, 20), 1, wx.EXPAND, 0)
        buttonRowSizer.Add(self.applyButton, 0, wx.ALIGN_BOTTOM, 0)
        buttonRowSizer.Add(self.cancelButton, 0, wx.ALIGN_BOTTOM, 0)
        buttonRowSizer.Add(self.okButton, 0, wx.ALIGN_BOTTOM, 0)
        mainSizer.Add(buttonRowSizer, 1, wx.ALL | wx.EXPAND, 2)
        self.SetSizer(mainSizer)
        mainSizer.Fit(self)
        self.Layout()

    def OnToggleSSL(self, event):
        if self.useSSLCheckBox.GetValue():
            if self.portTextCtrl.GetValue() == '143':
                self.portTextCtrl.SetValue('993')
        else:
            if self.portTextCtrl.GetValue() == '993':
                self.portTextCtrl.SetValue('143')


    def OnApplyClick(self, event):
        self.__copy_config()
        self.cfg.write()

    def OnOkClick(self, event):
        # check for all required fields
        counter = 0
        error = ''
        if self.fromAddressTextCtrl.GetValue() == "":
            counter += 1
            error += _('%d. From Address invalid\n') % counter
        if self.serverTextCtrl.GetValue() == "":
            counter += 1
            error += _('%d. Server incomplete\n') % counter
        if self.portTextCtrl.GetValue() == "" or re.search('\D', self.portTextCtrl.GetValue()):
            counter += 1
            error += _('%d. Port invalid\n') % counter
        if self.loginTextCtrl.GetValue() == "":
            counter += 1
            error += _('%d. Login incomplete\n') % counter
        if self.passwordTextCtrl.GetValue() == "":
            counter += 1
            error += _('%d. Password incomplete\n') % counter
        if  (not self.readCheckBox.GetValue() and
                not self.sentCheckBox.GetValue() and
                not self.unreadCheckBox.GetValue() and
                not self.unsentCheckBox.GetValue()):
            counter += 1
            error += _('%d. No messages to back-up selected. Please tick at least one of the states.') % counter

        if counter != 0:
            wx.MessageDialog(
                self,
                error,
                _('Incomplete'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
        else:
            self.__copy_config()
            event.Skip()

# end of class IMAPSettingsDialog



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Locales initialisation and gettext wrapper
'''

import os
import gettext
import locale
import codecs
import sys

_TRANSLATION = None


LOCAL_LOCALE_PATH = os.path.join('build', 'share', 'locale')
LOCALE_PATH = None

FALLBACK_LOCALE_CHARSET = 'iso-8859-1'

# Determine "correct" character set
try:
    # works only in python > 2.3
    LOCALE_CHARSET = locale.getpreferredencoding()
except:
    try:
        LOCALE_CHARSET = locale.getdefaultlocale()[1]
    except:
        try:
            LOCALE_CHARSET = sys.getdefaultencoding()
        except:
            LOCALE_CHARSET = FALLBACK_LOCALE_CHARSET
if LOCALE_CHARSET in [None, 'ANSI_X3.4-1968']:
    LOCALE_CHARSET = FALLBACK_LOCALE_CHARSET

try:
    CONSOLE_CHARSET = sys.stdout.encoding
except AttributeError:
    CONSOLE_CHARSET = None
if CONSOLE_CHARSET is None:
    CONSOLE_CHARSET = LOCALE_CHARSET
CONSOLE_ENCODER = codecs.getencoder(CONSOLE_CHARSET)

def ConsoleStrConv(txt):
    """
    This function coverts something (txt) to string form usable on console.
    """
    try:
        if isinstance(txt, str):
            return txt
        elif isinstance(txt, unicode):
            return str(CONSOLE_ENCODER(txt, 'replace')[0])
        return str(txt)
    except UnicodeEncodeError:
        return '???'

def StrConv(txt):
    """
    This function coverts something (txt) to string form usable by wxPython. There
    is problem that in default configuration in most distros (maybe all) default
    encoding for unicode objects is ascii. This leads to exception when converting
    something different than ascii. And this exception is not catched inside
    wxPython and leads to segfault.

    So if wxPython supports unicode, we give it unicode, otherwise locale
    dependant text.
    """
    try:
        if isinstance(txt, unicode):
            return txt
        elif isinstance(txt, str):
            return unicode(txt, LOCALE_CHARSET)
        return str(txt)
    except UnicodeEncodeError:
        return '???'

# detect html charset
HTML_CHARSET = LOCALE_CHARSET

# prepare html encoder
HTML_ENCODER = codecs.getencoder(HTML_CHARSET)

def HtmlStrConv(txt):
    """
    This function coverts something (txt) to string form usable by wxPython
    html widget. There is problem that in default configuration in most distros
    (maybe all) default encoding for unicode objects is ascii. This leads to
    exception when converting something different than ascii. And this
    exception is not catched inside wxPython and leads to segfault.

    So if wxPython supports unicode, we give it unicode, otherwise locale
    dependant text.
    """
    try:
        if isinstance(txt, unicode):
            return txt
        elif isinstance(txt, str):
            return unicode(txt, LOCALE_CHARSET)
        return str(txt)
    except UnicodeEncodeError:
        return '???'

def UnicodeConv(txt):
    """
    This function coverts something (txt) to string form usable by wxPython. There
    is problem that in default configuration in most distros (maybe all) default
    encoding for unicode objects is ascii. This leads to exception when converting
    something different than ascii. And this exception is not catched inside
    wxPython and leads to segfault.

    So if wxPython supports unicode, we give it unicode, otherwise locale
    dependant text.
    """
    try:
        if isinstance(txt, unicode):
            return txt
        elif isinstance(txt, str):
            return unicode(txt, LOCALE_CHARSET)
        return unicode(str(txt), LOCALE_CHARSET)
    except UnicodeEncodeError:
        return unicode('???')


def Init():
    '''
    Initialises gettext for wammu domain and installs global function _,
    which handles translations.
    '''
    global LOCALE_PATH
    switch = False
    if (os.path.exists('setup.py') and
        os.path.exists(LOCAL_LOCALE_PATH) and
        os.path.exists(
            os.path.join('Wammu', '__init__.py')
            )):
        LOCALE_PATH = LOCAL_LOCALE_PATH
        switch = True
    Install()
    if switch:
        print ConsoleStrConv(ugettext('Automatically switched to local locales.'))


def UseLocal():
    '''
    Use locales from current build dir.
    '''
    global LOCALE_PATH
    LOCALE_PATH = LOCAL_LOCALE_PATH
    Install()


def ngettext(msgid1, msgid2, n):
    if _TRANSLATION:
        return _TRANSLATION.ungettext(msgid1, msgid2, n)
    if n == 1:
        return msgid1
    else:
        return msgid2


def ugettext(message):
    if _TRANSLATION:
        return _TRANSLATION.ugettext(message)
    return message


def Install():
    global _TRANSLATION
    try:
        _TRANSLATION = gettext.translation(
            'wammu',
            localedir=LOCALE_PATH
        )
    except IOError:
        print ConsoleStrConv('Failed to load translation!')



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Generic thread wrapper used for reading things from phone
'''

import sys
import threading
import wx
import Wammu.Events
from Wammu.Locales import ugettext as _


class Thread(threading.Thread):
    def __init__(self, win, sm):
        threading.Thread.__init__(self)
        self.win = win
        self.sm = sm
        self.canceled = False

    def run(self):
        try:
            self.Run()
        except:
            evt = Wammu.Events.ExceptionEvent(data=sys.exc_info())
            wx.PostEvent(self.win, evt)
            self.ShowProgress(100)

    def Cancel(self):
        self.canceled = True

    def ShowError(self, info, finish=False):
        if finish:
            self.ShowProgress(100)
        lck = threading.Lock()
        lck.acquire()
        evt = Wammu.Events.ShowMessageEvent(
            message=Wammu.Utils.FormatError(
                _('Error while communicating with phone'),
                info),
            title=_('Error Occured'),
            type=wx.ICON_ERROR,
            errortype='gammu',
            lock=lck)
        wx.PostEvent(self.win, evt)
        lck.acquire()

    def ShowMessage(self, title, text):
        lck = threading.Lock()
        lck.acquire()
        evt = Wammu.Events.ShowMessageEvent(
            message=text,
            title=title,
            type=wx.ICON_INFORMATION,
            lock=lck)
        wx.PostEvent(self.win, evt)
        lck.acquire()

    def ShowProgress(self, progress):
        evt = Wammu.Events.ProgressEvent(
            progress=progress,
            cancel=self.Cancel)
        wx.PostEvent(self.win, evt)

    def SendData(self, datatype, data, last=True):
        evt = Wammu.Events.DataEvent(
            type=datatype,
            data=data,
            last=last)
        wx.PostEvent(self.win, evt)

    def Canceled(self):
        lck = threading.Lock()
        lck.acquire()
        evt = Wammu.Events.ShowMessageEvent(
            message=_('Action canceled by user!'),
            title=_('Action canceled'),
            type=wx.ICON_WARNING,
            lock=lck)
        wx.PostEvent(self.win, evt)
        lck.acquire()
        self.ShowProgress(100)



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Phone configuration wizard
'''

import wx
import wx.wizard
import Wammu.Paths
import Wammu.Wizard
import Wammu.Data
import Wammu.SettingsStorage
import Wammu.PhoneSearch
import Wammu.Events
import Wammu.Utils
import wx.lib.hyperlink
from Wammu.Locales import StrConv
from Wammu.Locales import ugettext as _


class FinalPage(Wammu.Wizard.InputPage):
    """
    Shows result of configuration, and allow to name phone.
    """
    def __init__(self, parent):
        Wammu.Wizard.InputPage.__init__(
            self, parent,
            _('Configuration done'),
            _('Thank you for configuring phone connection.'),
            parent.settings.GetName(),
            _('You can enter any name which you will use to identify your phone.')
        )

    def GetNext(self):
        return Wammu.Wizard.InputPage.GetNext(self)

    def Blocked(self, evt):
        self.parent.settings.SetName(self.edit.GetValue())
        return False

    def Activated(self, evt):
        self.edit.SetValue(self.parent.settings.GetName())

class TestPage(Wammu.Wizard.SimplePage):
    """
    Tests phone connection.
    """
    def __init__(self, parent):
        Wammu.Wizard.SimplePage.__init__(self, parent, _('Connection test'))
        self.detail = wx.StaticText(
                self,
                -1,
                _('Wammu is now testing phone connection, please wait...'))
        self.detail.Wrap(400)
        self.sizer.Add(self.detail, 0, wx.ALL, 5)
        self.name = ''
        self.thread = None
        self.Bind(Wammu.Events.EVT_DATA, self.OnSearchEnd)

    def GetNext(self):
        self.parent.pg_final.SetPrev(self)
        return self.parent.pg_final

    def Activated(self, evt):
        if evt.GetDirection():
            self.detail.SetLabel(
                    _('Wammu is now testing phone connection, please wait...'))
            device = self.parent.settings.GetPort()
            connection = self.parent.settings.GetGammuDriver()
            self.thread = Wammu.PhoneSearch.PhoneInfoThread(self, device, connection)
            self.thread.start()
            self.name = ''

    def OnSearchEnd(self, evt):
        self.thread = None
        if evt.data is None:
            self.detail.SetLabel('%s\n%s' % (
                    evt.error[0],
                    evt.error[1]
                    ))
            self.detail.Wrap(400)
        else:
            manuf = evt.data['Manufacturer']
            model = evt.data['Model'][0]
            self.name = '%s %s' % (manuf, model)
            self.parent.settings.SetName(self.name)
            self.detail.SetLabel(
                _('Phone has been found.') +
                (_('Manufacturer: %(manufacturer)s\nModel: %(model)s') % {
                    'manufacturer': manuf, 'model': model
                })
            )
            self.detail.Wrap(400)

    def Blocked(self, evt):
        if self.thread is not None and self.thread.isAlive():
            wx.MessageDialog(
                self,
                _('Phone connection test is still active, you can not continue.'),
                _('Testing still active!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            return True
        if evt.GetDirection() and self.name == '':
            dialog = wx.MessageDialog(
                self,
                _('Phone has not been found, are you sure you want to continue?'),
                _('Phone not found!'),
                wx.YES_NO | wx.NO_DEFAULT | wx.ICON_ERROR)
            if dialog.ShowModal() == wx.ID_YES:
                return False
            return True
        return False

    def Cancel(self, evt):
        # FIXME: we should abort test here
        if self.thread is not None and self.thread.isAlive():
            wx.MessageDialog(
                self,
                _('Phone connection test is still active, you can not continue.'),
                _('Testing still active!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            return False
        return True

class PhoneSearchPage(Wammu.Wizard.TextPage):
    """
    Search for phone.
    """
    def __init__(self, parent):
        Wammu.Wizard.TextPage.__init__(
            self, parent,
            _('Phone search'),
            _('Phone searching status') + ':'
        )
        self.Bind(Wammu.Events.EVT_DONE, self.OnDone)
        self.Bind(Wammu.Events.EVT_TEXT, self.OnText)
        self.Bind(Wammu.Events.EVT_SHOW_MESSAGE, self.OnShowMessage)
        self.results = []
        self.thread = None

    def GetNext(self):
        self.parent.pg_test.SetPrev(self)
        return self.parent.pg_test

    def Blocked(self, evt):
        if self.thread is not None and self.thread.isAlive():
            wx.MessageDialog(
                self,
                _('Phone search is still active, you can not continue.'),
                _('Searching still active!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            return True
        if evt.GetDirection() and len(self.results) == 0:
            wx.MessageDialog(
                self,
                _('No phone has not been found, you can not continue.'),
                _('No phone found!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            return True
        return False

    def Cancel(self, evt):
        # FIXME: we should abort searching here
        if self.thread is not None and self.thread.isAlive():
            wx.MessageDialog(
                self,
                _('Phone search is still active, you can not continue.'),
                _('Searching still active!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            return False
        return True

    def Activated(self, evt):
        if evt.GetDirection():
            self.edit.Clear()
            self.edit.AppendText(_('Wammu is now searching for phone:') + '\n')
            self.thread = Wammu.PhoneSearch.AllSearchThread(
                lock=False,
                callback=self.SearchDone,
                msgcallback=self.SearchMessage,
                noticecallback=self.SearchNotice,
                limit=self.parent.settings.GetConnection(),
                win=self
            )
            self.thread.start()
            self.results = []

    def SearchNotice(self, title, text):
        evt = Wammu.Events.ShowMessageEvent(
            message=text,
            title=title,
            type=wx.ICON_WARNING
        )
        wx.PostEvent(self, evt)

    def SearchMessage(self, text):
        """
        This has to send message as it is called from different thread.
        """
        evt = Wammu.Events.TextEvent(text=text + '\n')
        wx.PostEvent(self, evt)

    def SearchDone(self, lst):
        """
        This has to send message as it is called from different thread.
        """
        self.results = lst
        evt = Wammu.Events.DoneEvent()
        wx.PostEvent(self, evt)

    def OnText(self, evt):
        self.edit.AppendText(StrConv(evt.text))

    def OnShowMessage(self, evt):
        wx.MessageDialog(
            self.parent,
            StrConv(evt.message),
            StrConv(evt.title),
            wx.OK | evt.type
        ).ShowModal()

    def OnDone(self, evt):
        """
        Select one config to use.
        """
        if len(self.results) == 0:
            self.edit.AppendText(_('No phone has been found!') + '\n')
            return
        if len(self.results) > 1:
            # Allow user to select phone
            # FIXME: Might be in wizard, but this should be rare...
            choices = []
            for phone in self.results:
                choices.append(
                    _('Phone %(manufacturer)s %(model)s on device %(port)s using connection %(connection)s') %
                    {
                        'model': phone[2][0],
                        'manufacturer': phone[3],
                        'port': phone[0],
                        'connection': phone[1]
                    })
            dlg = wx.SingleChoiceDialog(
                self,
                _('Select phone to use from below list'),
                _('Select phone'),
                choices
            )
            if dlg.ShowModal() == wx.ID_OK:
                idx = dlg.GetSelection()
                config = self.results[idx]
            else:
                self.results = []
                config = None
        else:
            # Use directly only found phone
            config = self.results[0]

        if config is not None:
            self.parent.settings.SetPort(config[0])
            self.parent.settings.SetGammuDriver(config[1])
            self.edit.AppendText(_('Following phone will be used:') + '\n')
            self.edit.AppendText(
                _('Phone %(manufacturer)s %(model)s on device %(port)s using connection %(connection)s') %
                {
                    'model': config[2][0],
                    'manufacturer': config[3],
                    'port': config[0],
                    'connection': config[1]
                })
        else:
            self.edit.AppendText(_('No phone selected!') + '\n')


class ManualPage(Wammu.Wizard.MultiInputPage):
    """
    Manual phone configuration.
    """
    def __init__(self, parent):
        Wammu.Wizard.MultiInputPage.__init__(
            self, parent,
            _('Manual configuration'),
            [
                _('Device where phone is connected') + ':',
                _('Connection type') + ':',
            ],
            [
                parent.settings.GetDevices()[0],
                Wammu.Data.Connections,
            ]
        )

    def GetNext(self):
        self.parent.settings.SetPort(self.edits[0].GetValue())
        self.parent.settings.SetGammuDriver(self.edits[1].GetValue())
        self.parent.pg_test.SetPrev(self)
        return self.parent.pg_test

    def Blocked(self, evt):
        if evt.GetDirection():
            if self.edits[0].GetValue() == '':
                wx.MessageDialog(
                    self,
                    _('You need to select device which will be used.'),
                    _('No device selected!'),
                    wx.OK | wx.ICON_ERROR
                ).ShowModal()
                return True
            if self.edits[1].GetValue() == '':
                wx.MessageDialog(
                    self,
                    _('You need to select connection type which will be used.'),
                    _('No connection selected!'),
                    wx.OK | wx.ICON_ERROR
                ).ShowModal()
                return True
        return False

class PhonePortPage(Wammu.Wizard.InputPage):
    """
    Selects phone device.
    """
    def __init__(self, parent):
        ports, helptext = parent.settings.GetDevices()
        Wammu.Wizard.InputPage.__init__(
            self, parent,
            _('Phone Device'),
            _('Please enter device where phone is accessible') + ':',
            ports,
            helptext
        )

    def GetNext(self):
        self.parent.settings.SetPort(self.edit.GetValue())
        self.parent.pg_test.SetPrev(self)
        return self.parent.pg_test

    def Blocked(self, evt):
        if evt.GetDirection() and self.edit.GetValue() == '':
            wx.MessageDialog(
                self,
                _('You need to select device which will be used.'),
                _('No device selected!'),
                wx.OK | wx.ICON_ERROR
            ).ShowModal()
            return True
        return False

class PhoneGammuDriverPage(Wammu.Wizard.ChoicePage):
    """
    Selects real Gammu phone driver.
    """
    def __init__(self, parent):
        self.names, connections, helps = parent.settings.GetGammuDrivers()

        if len(self.names) == 0:
            Wammu.Wizard.SimplePage.__init__(
                self, parent,
                _('Driver to use'),
                _('Sorry no driver matches your configuration, please return back and try different settings or manual configuration.')
            )
        else:
            Wammu.Wizard.ChoicePage.__init__(
                self, parent,
                _('Driver to use'),
                _('Driver to use'),
                connections, helps,
                extratext=_('Please select which driver you want to use. Follow the help text shown below to select the best one.')
            )

    def GetNext(self):
        """
        Dynamically create next page for current settings.
        """
        if len(self.names) == 0:
            return None
        self.parent.settings.SetGammuDriver(self.names[self.GetType()])
        next = PhonePortPage(self.parent)
        next.SetPrev(self)
        return next


class PhoneDriverPage(Wammu.Wizard.ChoicePage):
    """
    Selects Gammu phone driver type.
    """
    def __init__(self, parent):
        self.names, connections, helps = parent.settings.GetDrivers()

        Wammu.Wizard.ChoicePage.__init__(
            self,
            parent,
            _('Connection type'),
            _('Connection type'),
            connections, helps,
            extratext=_('Please select connection type, default choice should be best in most cases.')
        )

    def GetNext(self):
        """
        Dynamically create next page for current settings.
        """
        self.parent.settings.SetDriver(self.names[self.GetType()])
        next = PhoneGammuDriverPage(self.parent)
        next.SetPrev(self)
        return next


class PhoneManufacturerPage(Wammu.Wizard.ChoicePage):
    """
    Selects phone manufacturer.
    """
    def __init__(self, parent):
        self.names, connections, helps = parent.settings.GetManufacturers()

        Wammu.Wizard.ChoicePage.__init__(
            self, parent,
            _('Phone type'),
            _('Phone type'),
            connections, helps,
            extratext=_('Please select phone manufacturer or type. Try to be as specific as possible.'),
        )

    def GetNext(self):
        """
        Dynamically create next page for current settings.
        """
        self.parent.settings.SetManufacturer(self.names[self.GetType()])
        next = PhoneDriverPage(self.parent)
        next.SetPrev(self)
        return next

class PhoneConnectionPage(Wammu.Wizard.ChoicePage):
    """
    Selects phone connection type.
    """
    def __init__(self, parent, search=True):
        self.names = []
        self.search = search
        connections = []
        helps = []

        if search:
            self.names.append('all')
            connections.append(_('Search all connections'))
            helps.append(_('Wizard will search for all possible connections. It might take quite long time to search all possible connection types.'))

        self.names.append('usb')
        connections.append(_('USB cable'))
        helps.append(_('Many phones now come with USB cable, select this if you\'re using this connection type.'))

        self.names.append('bluetooth')
        connections.append(_('Bluetooth'))
        helps.append(_('Bluetooth connection is wireless and does not require direct visibility. Phone needs to be properly paired with computer before proceeding.'))

        self.names.append('irda')
        connections.append(_('IrDA'))
        helps.append(_('IrDA wireless connection requires direct visibility, please make sure this is fullfilled and computer can see phone.'))

        self.names.append('serial')
        connections.append(_('Serial cable'))
        helps.append(_('This is not often used connection, but was very popular for older phones.'))

        Wammu.Wizard.ChoicePage.__init__(
            self, parent,
            _('Connection type'),
            _('Connection type'),
            connections, helps,
            extratext=_('How is your phone connected?'),
        )

    def GetNext(self):
        self.parent.settings.SetConnection(self.names[self.GetType()])
        return Wammu.Wizard.ChoicePage.GetNext(self)

class ConfigTypePage(Wammu.Wizard.ChoicePage):
    """
    Allows user to select how to configure phone.
    """
    def __init__(self, parent, pg0, pg1, pg2):
        Wammu.Wizard.ChoicePage.__init__(
            self, parent,
            _('Configuration style'),
            _('Configuration style'),
            [
                _('Guided configuration'),
                _('Automatically search for a phone'),
                _('Manual configuration'),
            ],

            [
                _('You will be guided through configuration by phone connection type and vendor.'),
                _('Wizard will attempt to search phone on usual ports.'),
                _('You know what you are doing and know exact parameters you need for connecting to phone.'),
            ],
            [pg0, pg1, pg2],
            extratext=_('How do you want to configure your phone connection?'),
        )
        self.info = wx.StaticText(
            self,
            -1,
            _('If you have no idea how to configure your phone connection, you can look at Gammu Phone Database for other users experiences:'))
        self.info.Wrap(400)
        self.sizer.Add(self.info, 0, wx.ALL, 5)
        self.link = wx.lib.hyperlink.HyperLinkCtrl(
                self,
                -1,
                'http://%scihar.com/gammu/phonedb' % Wammu.Utils.GetWebsiteLang())
        self.sizer.Add(self.link, 0, wx.ALL, 5)

class WelcomePage(Wammu.Wizard.SimplePage):
    """
    First page of Wizard.
    """
    def __init__(self, parent):
        Wammu.Wizard.SimplePage.__init__(
            self, parent,
            _('Welcome'),
            _('This wizard will help you with configuring phone connection in Wammu.'),
            [
                '',
                _('Please make sure you have phone ready, powered on and one of connection methods is set up:'),
                '  - %s' % _('Cable is connected.'),
                '  - %s' % _('You have enabled IrDA and phone is in visible range.'),
                '  - %s' % _('You have paired Bluetooth with computer.'),
                '',
                _('As soon as your phone is ready, you can continue.'),
            ]
        )

class ConfigureWizard:
    def __init__(self, parent, position=0):
        bmp = wx.Bitmap(Wammu.Paths.MiscPath('phonewizard'))
        self.wiz = wx.wizard.Wizard(
                parent,
                -1,
                _('Wammu Phone Configuration Wizard'),
                bmp)
        self.wiz.settings = Wammu.SettingsStorage.Settings()
        self.wiz.settings.SetPosition(position)

        self.wiz.Bind(wx.wizard.EVT_WIZARD_PAGE_CHANGING, self.OnPageChanging)
        self.wiz.Bind(wx.wizard.EVT_WIZARD_PAGE_CHANGED, self.OnPageChanged)
        self.wiz.Bind(wx.wizard.EVT_WIZARD_CANCEL, self.OnCancel)

        # Create pages
        self.pg_title = WelcomePage(self.wiz)

        self.pg_search1 = PhoneConnectionPage(self.wiz)
        self.pg_search2 = PhoneSearchPage(self.wiz)

        self.pg_guide1 = PhoneConnectionPage(self.wiz, False)
        self.pg_guide2 = PhoneManufacturerPage(self.wiz)

        self.pg_manual1 = ManualPage(self.wiz)

        self.pg_type = ConfigTypePage(
                self.wiz,
                self.pg_guide1,
                self.pg_search1,
                self.pg_manual1)

        self.pg_final = FinalPage(self.wiz)
        self.pg_test = TestPage(self.wiz)
        self.wiz.pg_final = self.pg_final
        self.wiz.pg_test = self.pg_test

        # Set their order
        self.pg_title.SetNext(self.pg_type)
        self.pg_type.SetPrev(self.pg_title)

        self.pg_type.SetNext(self.pg_search1) # overrided by it's GetNext

        # Set previous page for all types
        self.pg_search1.SetPrev(self.pg_type)
        self.pg_guide1.SetPrev(self.pg_type)
        self.pg_manual1.SetPrev(self.pg_type)

        self.pg_guide1.SetNext(self.pg_guide2)
        self.pg_guide2.SetPrev(self.pg_guide1)
        # rest of guide is created dynamically

        self.pg_search1.SetNext(self.pg_search2)
        self.pg_search2.SetPrev(self.pg_search1)
        # rest of search is created dynamically

        # Resize wizard
        self.wiz.FitToPage(self.pg_title)

    def OnPageChanging(self, evt):
        if evt.GetPage().Blocked(evt):
            evt.Veto()

    def OnPageChanged(self, evt):
        evt.GetPage().Activated(evt)

    def OnCancel(self, evt):
        if not evt.GetPage().Cancel(evt):
            evt.Veto()

    def Run(self):
        return self.wiz.RunWizard(self.pg_title)

    def Execute(self):
        if self.Run():
            return self.wiz.settings.GetSettings()
        else:
            return None


def RunConfigureWizard(parent, position=0):
    """
    Executes wizard for configuring phone
    """
    return ConfigureWizard(parent, position).Execute()

class WizardApp(wx.App):
    def OnInit(self):

        self.SetAppName('Wammu Phone Configuration Wizard')
        vendor = StrConv(u'Michal Čihař')
        if vendor.find('?') != -1:
            vendor = 'Michal Čihař'
        self.SetVendorName(vendor)

        # Return a success flag
        return True



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
webbrowser wrapper
'''

import webbrowser
import threading


class BrowserThread(threading.Thread):
    '''
    Thread which opens URL in browser.
    '''

    def __init__(self, url):
        '''
        Creates new BrowserThread object.

        @param url: URL to open when thread is started.
        '''
        threading.Thread.__init__(self)
        self._url = url

    def run(self):
        '''
        Actually opens browser.
        '''
        webbrowser.open(self._url)


def Open(url):
    '''
    Convenience wrapper to open URL in browser without blocking main
    thread.
    '''
    BrowserThread(url).start()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Execution script for configuration
'''

import os
import sys
import getopt
import wx
import time
import Wammu
import Wammu.GammuSettings
import Wammu.Locales
from Wammu.Locales import ugettext as _


def version():
    '''
    Displays version information.
    '''
    print (_('Wammu Configurator - Wammu and Gammu configurator version %s')
           % Wammu.__version__)


def usage():
    '''
    Shows program usage.
    '''
    version()
    print _('Usage: %s [OPTION...]' % os.path.basename(__file__))
    print
    print _('Options:')
    print '%-20s ... %s' % (
        '-h/--help',
        _('show this help')
    )
    print '%-20s ... %s' % (
        '-v/--version',
        _('show program version')
    )
    print '%-20s ... %s' % (
        '-l/--local-locales',
        _('force using of locales from current directory rather than system ones')
    )
    print


def parse_options():
    '''
    Processes program options.
    '''
    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            'hvl',
            ['help', 'version', 'local-locales']
        )
    except getopt.GetoptError, val:
        usage()
        print _('Command line parsing failed with error:')
        print val
        sys.exit(2)

    if len(args) != 0:
        usage()
        print _('Extra unrecognized parameters passed to program')
        sys.exit(3)

    for opt, dummy in opts:
        if opt in ('-l', '--local-locales'):
            Wammu.Locales.UseLocal()
            print _('Using local built locales!')
        if opt in ('-h', '--help'):
            usage()
            sys.exit()
        if opt in ('-v', '--version'):
            version()
            sys.exit()


def do_wizard():
    '''
    Runs configuration wizard.
    '''
    app = Wammu.PhoneWizard.WizardApp()

    wammu_cfg = Wammu.WammuSettings.WammuConfig()

    config = Wammu.GammuSettings.GammuSettings(wammu_cfg)

    position = config.SelectConfig(new=True)

    if position is None:
        sys.exit()

    result = Wammu.PhoneWizard.RunConfigureWizard(None, position)
    if result is not None:
        busy = wx.BusyInfo(_('Updating gammu configuration...'))
        time.sleep(0.1)
        wx.Yield()
        config.SetConfig(
            result['Position'],
            result['Device'],
            result['Connection'],
            result['Name']
        )
        del busy
    app.Destroy()


if __name__ == '__main__':
    Wammu.Locales.Init()
    parse_options()
    # need to be imported after locales are initialised
    import Wammu.PhoneWizard
    import Wammu.WammuSettings
    do_wizard()



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Calendar reader
'''

import Wammu.Reader
import Wammu.Utils
import Wammu


class GetCalendar(Wammu.Reader.Reader):
    def GetStatus(self):
        status = self.sm.GetCalendarStatus()
        return status['Used']

    def GetNextStart(self):
        return self.sm.GetNextCalendar(Start=True)

    def GetNext(self, location):
        return self.sm.GetNextCalendar(Location=location)

    def Get(self, location):
        return self.sm.GetCalendar(Location=location)

    def Parse(self, value):
        Wammu.Utils.ParseCalendar(value)

    def Send(self, data):
        self.SendData(['calendar', '  '], data)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Execution script
'''

import os
import sys
import getopt
import Wammu
import Wammu.Locales
from Wammu.Locales import ugettext as _

# Disable warning about missing files
# This can be caused by attempt to import Python modules, which don't
# have all DLLs satisfied.
if sys.platform.startswith('win'):
    import win32api
    import win32con
    win32api.SetErrorMode(win32con.SEM_NOOPENFILEERRORBOX)


def version():
    '''
    Displays version information.
    '''
    print _('Wammu - Windowed Gammu version %s') % Wammu.__version__


def usage():
    '''
    Shows program usage.
    '''
    version()
    print _('Usage: %s [OPTION...]' % os.path.basename(__file__))
    print
    print _('Options:')
    print '%-20s ... %s' % (
        '-h/--help',
        _('show this help')
    )
    print '%-20s ... %s' % (
        '-v/--version',
        _('show program version')
    )
    print '%-20s ... %s' % (
        '-l/--local-locales',
        _('force using of locales from current directory rather than system ones')
    )
    print '%-20s ... %s' % (
        '-i/--info',
        _('prints connection settings and tries to connect the phone')
    )
    print '%-20s ... %s' % (
        '-d/--debug',
        _('enables debug output to stderr')
    )
    print


def info():
    '''
    Displays configuration summary and tries to connect to phone.
    '''
    import Wammu.WammuSettings
    import Wammu.Utils
    import gammu

    settings = Wammu.WammuSettings.WammuConfig()
    section = settings.ReadInt('/Gammu/Section')
    config = settings.gammu.GetConfig(section)
    if config['Connection'] == '' or config['Device'] == '':
        print _('Wammu is not configured!')

    cfg = {
        'StartInfo': settings.ReadBool('/Gammu/StartInfo'),
        'UseGlobalDebugFile': True,
        'DebugFile': None,  # Set on other place
        'SyncTime': settings.ReadBool('/Gammu/SyncTime'),
        'Connection': config['Connection'],
        'LockDevice': settings.ReadBool('/Gammu/LockDevice'),
        'DebugLevel': 'textalldate',  # Set on other place
        'Device': config['Device'],
        'Model': config['Model'],
    }

    # Compatibility with old Gammu versions
    cfg = Wammu.Utils.CompatConfig(cfg)

    print _('Wammu configuration:')
    print '%-15s: %s' % (_('Connection'), cfg['Connection'])
    print '%-15s: %s' % (_('Model'), cfg['Model'])
    print '%-15s: %s' % (_('Device'), cfg['Device'])
    print _('Connecting...')
    if Wammu.debug:
        gammu.SetDebugFile(sys.stderr)
        gammu.SetDebugLevel('textalldate')
    sm = gammu.StateMachine()
    sm.SetConfig(0, cfg)
    sm.Init()
    print _('Getting phone information...')
    Manufacturer = sm.GetManufacturer()
    Model = sm.GetModel()
    IMEI = sm.GetIMEI()
    Firmware = sm.GetFirmware()
    Code = sm.GetSecurityStatus()
    print _('Phone infomation:')
    print '%-15s: %s' % (_('Manufacturer'), Manufacturer)
    print '%-15s: %s (%s)' % (_('Model'), Model[0], Model[1])
    print '%-15s: %s' % (_('IMEI'), IMEI)
    print '%-15s: %s' % (_('Firmware'), Firmware[0])
    if Code is not None:
        print '%-15s: %s' % (_('Requested code'), Code)


def parse_options():
    '''
    Processes program options.
    '''
    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            'hvlid',
            ['help', 'version', 'local-locales', 'info', 'debug']
        )
    except getopt.GetoptError, val:
        usage()
        print _('Command line parsing failed with error:')
        print val
        sys.exit(2)

    if len(args) != 0:
        usage()
        print _('Extra unrecognized parameters passed to program')
        sys.exit(3)

    do_info = False

    for opt, dummy in opts:
        if opt in ('-l', '--local-locales'):
            Wammu.Locales.UseLocal()
            print _('Using local built locales!')
        if opt in ('-h', '--help'):
            usage()
            sys.exit()
        if opt in ('-v', '--version'):
            version()
            sys.exit()
        if opt in ('-i', '--info'):
            do_info = True
        if opt in ('-d', '--debug'):
            Wammu.debug = True

    if do_info:
        info()
        sys.exit()


if __name__ == '__main__':
    Wammu.Locales.Init()
    parse_options()
    # need to be imported after locales are initialised
    import Wammu.App
    Wammu.App.Run()



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Dialog for editing phone numbers list.

Created with help of wxGlade.
'''

import os
import wx
import Wammu.Select
import Wammu.Utils
import Wammu.PhoneValidator
from Wammu.Locales import StrConv
from Wammu.Locales import ugettext as _


class EditContactList(wx.Dialog):
    def __init__(self, parent, contacts, current, *args, **kwds):
        self.contacts = contacts
        self.current = current
        kwds['style'] = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
        wx.Dialog.__init__(self, parent, *args, **kwds)
        self.__init_data()
        self.all_label = wx.StaticText(self, -1, _('Available contacts:'))
        self.all_contacts = wx.ListBox(self, -1, choices=self.optionslist, style=wx.LB_EXTENDED)
        self.add_button = wx.Button(self, wx.ID_ADD)
        self.delete_button = wx.Button(self, wx.ID_DELETE)
        self.current_label = wx.StaticText(self, -1, _('Current recipients:'))
        self.current_contacts = wx.ListBox(self, -1, choices=self.currentlist, style=wx.LB_EXTENDED)
        self.save_button = wx.Button(self, wx.ID_SAVEAS)
        # TODO: Load would be better
        self.load_button = wx.Button(self, wx.ID_OPEN)

        self.button_sizer = wx.StdDialogButtonSizer()
        self.button_sizer.AddButton(wx.Button(self, wx.ID_OK))
        self.button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))

        self.__set_properties()
        self.__do_layout()
        self.__bind_events()

    def __bind_events(self):
        self.Bind(wx.EVT_BUTTON, self.Add, self.add_button)
        self.Bind(wx.EVT_BUTTON, self.Delete, self.delete_button)
        self.Bind(wx.EVT_BUTTON, self.Save, self.save_button)
        self.Bind(wx.EVT_BUTTON, self.Load, self.load_button)

    def __init_data(self):
        self.numberlist = []
        self.optionslist = []

        for item in self.contacts:
            numbers = []
            texts = []
            prefix = ''
            if item['Name'] != '':
                prefix = '%s: ' % item['Name']
            for i in range(len(item['Entries'])):
                if Wammu.Utils.GetItemType(item['Entries'][i]['Type']) == 'phone':
                    numbers.append(item['Entries'][i]['Value'])
                    texts.append(StrConv('%s (%s)' % (item['Entries'][i]['Value'], item['Entries'][i]['Type'])))

            if len(numbers) == 0:
                continue

            for x in range(len(numbers)):
                self.numberlist.append(numbers[x])
                self.optionslist.append(prefix + texts[x])

        self.currentlist = Wammu.PhoneValidator.SplitNumbers(self.current)
        self.wildcard = ''
        self.wildcard += _('Contact list') + ' (*.contactlist)|*.contactlist|'
        self.wildcard += _('All files') + ' (*.*)|*.*'

    def __set_properties(self):
        self.SetTitle(_('Edit contacts list'))

    def __do_layout(self):
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add((10, 10), 0, wx.ADJUST_MINSIZE, 0)
        sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_2.Add((10, 10), 0, wx.ADJUST_MINSIZE, 0)
        sizer_4 = wx.BoxSizer(wx.VERTICAL)
        sizer_5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6 = wx.BoxSizer(wx.VERTICAL)
        sizer_3 = wx.BoxSizer(wx.VERTICAL)
        sizer_6.Add(self.all_label, 0, wx.BOTTOM | wx.ADJUST_MINSIZE, 5)
        sizer_6.Add(self.all_contacts, 1, wx.EXPAND | wx.ADJUST_MINSIZE, 0)
        sizer_2.Add(sizer_6, 1, wx.ALL | wx.EXPAND | wx.ADJUST_MINSIZE, 0)
        sizer_3.Add(self.add_button, 0, wx.ADJUST_MINSIZE, 0)
        sizer_3.Add((20, 20), 0, wx.EXPAND | wx.ADJUST_MINSIZE, 0)
        sizer_3.Add(self.delete_button, 0, wx.ADJUST_MINSIZE, 0)
        sizer_2.Add(sizer_3, 0, wx.ALL | wx.EXPAND | wx.ADJUST_MINSIZE, 10)
        sizer_4.Add(self.current_label, 0, wx.BOTTOM | wx.ADJUST_MINSIZE, 5)
        sizer_4.Add(self.current_contacts, 1, wx.EXPAND | wx.ADJUST_MINSIZE, 0)
        sizer_5.Add(self.save_button, 0, wx.ALL | wx.ADJUST_MINSIZE, 5)
        sizer_5.Add(self.load_button, 0, wx.ALL | wx.ADJUST_MINSIZE, 5)
        sizer_4.Add(sizer_5, 0, wx.EXPAND, 0)
        sizer_2.Add(sizer_4, 1, wx.EXPAND, 0)
        sizer_2.Add((10, 10), 0, wx.ADJUST_MINSIZE, 0)
        sizer_1.Add(sizer_2, 1, wx.EXPAND, 0)
        self.button_sizer.Realize()
        sizer_1.Add(self.button_sizer, 0, wx.ALIGN_RIGHT, 0)
        self.SetAutoLayout(True)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        sizer_1.SetSizeHints(self)
        self.Layout()

    def GetNumbers(self):
        return ' '.join(self.currentlist)

    def Add(self, evt=None):
        index = self.all_contacts.GetSelections()
        if index == wx.NOT_FOUND:
            return
        for i in index:
            newone = self.numberlist[i]
            if newone not in self.currentlist:
                self.currentlist.append(newone)
                self.current_contacts.Append(newone)

    def Delete(self, evt=None):
        index = self.current_contacts.GetSelections()
        if index == wx.NOT_FOUND:
            return
        for i in reversed(index):
            del self.currentlist[i]
            self.current_contacts.Delete(i)

    def Save(self, evt=None):
        dlg = wx.FileDialog(
            self,
            _('Load contacts from file'),
            os.getcwd(),
            '',
            self.wildcard,
            wx.SAVE | wx.OVERWRITE_PROMPT | wx.CHANGE_DIR
        )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            try:
                data = file(path, 'w')
                for line in self.currentlist:
                    data.write('%s\n' % line)
                data.close()
            except IOError:
                wx.MessageDialog(
                    self,
                    _('Selected file "%s" could not be written.') % path,
                    _('File can not be created!'),
                    wx.OK | wx.ICON_ERROR
                ).ShowModal()

    def Load(self, evt=None):
        dlg = wx.FileDialog(
            self,
            _('Load contacts from file'),
            os.getcwd(),
            '',
            self.wildcard,
            wx.OPEN | wx.CHANGE_DIR
        )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            try:
                newlist = []
                data = file(path, 'r')
                for line in data:
                    newlist.append(line.strip())
                data.close()
                self.currentlist = newlist
                self.current_contacts.Set(newlist)
            except IOError:
                wx.MessageDialog(
                    self,
                    _('Selected file "%s" was not found, no data read.') % path,
                    _('File not found!'),
                    wx.OK | wx.ICON_ERROR
                ).ShowModal()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Written by Martin v. Löwis <loewis@informatik.hu-berlin.de>
# Plural forms support added by alexander smishlajev <alex@tycobka.lv>

"""Generate binary message catalog from textual translation description.

This program converts a textual Uniforum-style message catalog (.po file) into
a binary GNU catalog (.mo file).  This is essentially the same function as the
GNU msgfmt program, however, it is a simpler implementation.

Usage: msgfmt.py [OPTIONS] filename.po

Options:
    -o file
    --output-file=file
        Specify the output file to write to.  If omitted, output will go to a
        file named filename.mo (based off the input file name).

    -h
    --help
        Print this message and exit.

    -V
    --version
        Display version information and exit.
"""

import os
import sys
import ast
import getopt
import struct
import array

__version__ = "1.1"

MESSAGES = {}


def _(text):
    return text

# Just a hack to translate desktop file
# l10n: Name of program shown in desktop file
DESKTOP_NAME = _('Wammu')
# l10n: Generic name of program shown in desktop file
DESKTOP_GENERIC_NAME = _('Mobile Phone Manager')
# l10n: Comment about program shown in desktop file
DESKTOP_COMMENT = _('Application for mobile phones - frontend for Gammu')
# l10n: Search terms to find this application. Do NOT translate or localize the semicolons! The list MUST also end with a semicolon!
DESKTOP_KEYWORDS = _('phone;mobile;sms;contact;calendar;todo;')

DESKTOP_DESCRIPTION_1 = _(
    'Wammu is a mobile phone manager that uses Gammu as its backend. It works '
    'with any phone that Gammu supports, including many models from Nokia, '
    'Siemens, and Alcatel.'
)
DESKTOP_DESCRIPTION_2 = _(
    'It has complete support (read, edit, delete, copy) for contacts, todo, '
    'and calendar. It can read, save, and send SMS. It includes an SMS '
    'composer for multi-part SMS messages, and it can display SMS messages '
    'that include pictures. Currently, only text and predefined bitmaps or '
    'sounds can be edited in the SMS composer. It can export messages to an '
    'IMAP4 server (or other email storage).'
)

DESKTOP_TRANSLATIONS = {}


def usage(code, msg=''):
    print >> sys.stderr, __doc__
    if msg:
        print >> sys.stderr, msg
    sys.exit(code)


def add(id, str, fuzzy):
    "Add a non-fuzzy translation to the dictionary."
    if not fuzzy and str and not str.startswith('\0'):
        MESSAGES[id] = str
        if id == DESKTOP_NAME:
            DESKTOP_TRANSLATIONS['Name'] = str
        elif id == DESKTOP_GENERIC_NAME:
            DESKTOP_TRANSLATIONS['GenericName'] = str
        elif id == DESKTOP_COMMENT:
            DESKTOP_TRANSLATIONS['Comment'] = str
        elif id == DESKTOP_KEYWORDS:
            DESKTOP_TRANSLATIONS['Keywords'] = str
        elif id == DESKTOP_DESCRIPTION_1:
            DESKTOP_TRANSLATIONS['Description_1'] = str
        elif id == DESKTOP_DESCRIPTION_2:
            DESKTOP_TRANSLATIONS['Description_2'] = str


def generate():
    "Return the generated output."
    keys = MESSAGES.keys()
    # the keys are sorted in the .mo file
    keys.sort()
    offsets = []
    ids = strs = ''
    for id in keys:
        # For each string, we need size and file offset.  Each string is NUL
        # terminated; the NUL does not count into the size.
        offsets.append((len(ids), len(id), len(strs), len(MESSAGES[id])))
        ids += id + '\0'
        strs += MESSAGES[id] + '\0'
    output = ''
    # The header is 7 32-bit unsigned integers.  We don't use hash tables, so
    # the keys start right after the index tables.
    # translated string.
    keystart = 7*4 + 16*len(keys)
    # and the values start after the keys
    valuestart = keystart + len(ids)
    koffsets = []
    voffsets = []
    # The string table first has the list of keys, then the list of values.
    # Each entry has first the size of the string, then the file offset.
    for o1, l1, o2, l2 in offsets:
        koffsets += [l1, o1 + keystart]
        voffsets += [l2, o2 + valuestart]
    offsets = koffsets + voffsets
    output = struct.pack("Iiiiiii",
                         0x950412deL,       # Magic
                         0,                 # Version
                         len(keys),         # # of entries
                         7*4,               # start of key index
                         7*4+len(keys)*8,   # start of value index
                         0, 0)              # size and offset of hash table
    output += array.array("i", offsets).tostring()
    output += ids
    output += strs
    return output


def make(filename, outfile):
    ID = 1
    STR = 2
    global MESSAGES
    global DESKTOP_TRANSLATIONS
    MESSAGES = {}
    DESKTOP_TRANSLATIONS = {}

    # Compute .mo name from .po name and arguments
    if filename.endswith('.po'):
        infile = filename
    else:
        infile = filename + '.po'
    if outfile is None:
        outfile = os.path.splitext(infile)[0] + '.mo'

    try:
        lines = open(infile).readlines()
    except IOError, msg:
        print >> sys.stderr, msg
        sys.exit(1)

    section = None
    fuzzy = 0
    msgid = msgstr = ''

    # Parse the catalog
    lno = 0
    for l in lines:
        lno += 1
        # If we get a comment line after a msgstr, this is a new entry
        if l[0] == '#' and section == STR:
            add(msgid, msgstr, fuzzy)
            section = None
            fuzzy = 0
        # Record a fuzzy mark
        if l[:2] == '#,' and (l.find('fuzzy') >= 0):
            fuzzy = 1
        # Skip comments
        if l[0] == '#':
            continue
        # Start of msgid_plural section, separate from singular form with \0
        if l.startswith('msgid_plural'):
            msgid += '\0'
            l = l[12:]
        # Now we are in a msgid section, output previous section
        elif l.startswith('msgid'):
            if section == STR:
                add(msgid, msgstr, fuzzy)
            section = ID
            l = l[5:]
            msgid = msgstr = ''
        # Now we are in a msgstr section
        elif l.startswith('msgstr'):
            section = STR
            l = l[6:]
            # Check for plural forms
            if l.startswith('['):
                # Separate plural forms with \0
                if not l.startswith('[0]'):
                    msgstr += '\0'
                # Ignore the index - must come in sequence
                l = l[l.index(']') + 1:]
        # Skip empty lines
        l = l.strip()
        if not l:
            continue
        l = ast.literal_eval(l)
        if section == ID:
            msgid += l
        elif section == STR:
            msgstr += l
        else:
            print >> sys.stderr, 'Syntax error on %s:%d' % (infile, lno), \
                  'before:'
            print >> sys.stderr, l
            sys.exit(1)
    # Add last entry
    if section == STR:
        add(msgid, msgstr, fuzzy)

    # Compute output
    output = generate()

    try:
        open(outfile, "wb").write(output)
    except IOError as msg:
        print >> sys.stderr, msg


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'hVo:',
                                   ['help', 'version', 'output-file='])
    except getopt.error, msg:
        usage(1, msg)

    outfile = None
    # parse options
    for opt, arg in opts:
        if opt in ('-h', '--help'):
            usage(0)
        elif opt in ('-V', '--version'):
            print >> sys.stderr, "msgfmt.py", __version__
            sys.exit(0)
        elif opt in ('-o', '--output-file'):
            outfile = arg
    # do it
    if not args:
        print >> sys.stderr, 'No input file given'
        print >> sys.stderr, "Try `msgfmt --help' for more information."
        return

    for filename in args:
        make(filename, outfile)


if __name__ == '__main__':
    main()



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Data path definition and misc path functions
'''

import os
import os.path
import sys

POSSIBLE_PATHS = [
    os.path.join(sys.exec_prefix, 'share', 'Wammu'), # systemwide
    os.path.join(sys.exec_prefix, 'local', 'share', 'Wammu'), # systemwide, default distutils
    os.path.join(os.path.dirname(__file__), '..'), # Local directory
]

def CheckImagesPath(path):
    return os.path.exists(os.path.join(path, 'images'))

for DATAPATH in POSSIBLE_PATHS:
    if CheckImagesPath(DATAPATH):
        break

if not CheckImagesPath(DATAPATH):
    print 'Could not find images, you will not see them, check your installation!'

def AppIconPath(*args):
    if sys.platform == 'win32':
        ext = 'ico'
    else:
        ext = 'png'
    p = os.path.join(*args) + os.extsep + ext
    if os.path.exists(os.path.join(sys.exec_prefix, 'share', 'pixmaps', p)):
        return os.path.join(sys.exec_prefix, 'share', 'pixmaps', p)
    elif os.path.exists(os.path.join(DATAPATH, '..', 'pixmaps', p)):
        return os.path.join(DATAPATH, '..', 'pixmaps', p)
    elif os.path.exists(os.path.join('usr', 'share', 'pixmaps', p)):
        return os.path.join('usr', 'share', 'pixmaps', p)
    else:
        return os.path.join('.', 'icon', p)

def IconPath(*args):
    return ImagePath('icons', *args)

def MiscPath(*args):
    return ImagePath('misc', *args)

def ImagePath(*args):
    return os.path.join(DATAPATH, 'images', *args) + os.extsep + 'png'



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Error messages
'''

import wx
import os
import Wammu
import Wammu.Webbrowser
import Wammu.Paths
import Wammu.ErrorLog
from Wammu.Locales import ugettext as _


BUG_SEARCH_URL = 'https://github.com/search?l=&q=%s+%%40gammu&ref=advsearch&type=Issues'
BUG_REPORT_URL = 'https://github.com/gammu/wammu/issues/new'

class ErrorMessage(wx.Dialog):
    '''
    Error message box with support for saving debug log and reporting
    bug to https://github.com/gammu/wammu/issues.
    '''
    def __init__(self, parent, message, title, traceid=None,
                 autolog=None, exception=None):
        wx.Dialog.__init__(self, parent, -1, title)

        sizer = wx.BoxSizer(wx.VERTICAL)
        textsizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(textsizer, flag=wx.ALL, border=10)

        bitmap = wx.Bitmap(Wammu.Paths.MiscPath('error'))
        icon = wx.StaticBitmap(
            self, -1, bitmap,
            size=(bitmap.GetWidth(), bitmap.GetHeight())
        )
        textsizer.Add(icon, flag=wx.RIGHT, border=10)

        if exception is not None:
            message += '\n\n'
            message += exception
        if autolog is not None:
            message += '\n\n'
            message += (
                    _('Debug log has been automatically saved to %s, you are strongly encouraged to include it in bugreport.')
                    % autolog)
        msg = wx.TextCtrl(self, style=wx.TE_BESTWRAP|wx.TE_MULTILINE|wx.TE_READONLY|wx.BORDER_NONE, size=(500, 400))
        msg.SetValue(message)
        msg.SetBackgroundColour(wx.SystemSettings.GetColour(4))
        textsizer.Add(msg)

        buttonsizer = wx.StdDialogButtonSizer()
        buttonsizer.AddButton(wx.Button(self, wx.ID_OK))

        if traceid is None:
            savebutton = wx.Button(self, -1, _('Save debug log...'))
            buttonsizer.SetCancelButton(savebutton)
            self.Bind(wx.EVT_BUTTON, self.OnSave, savebutton)
        else:
            self.traceid = traceid
            searchbutton = wx.Button(self, -1, _('Search for similar reports'))
            buttonsizer.SetCancelButton(searchbutton)
            self.Bind(wx.EVT_BUTTON, self.OnSearch, searchbutton)

        self.reportbutton = wx.Button(self, -1, _('Report bug'))
        buttonsizer.SetNegativeButton(self.reportbutton)
        self.Bind(wx.EVT_BUTTON, self.OnReport, self.reportbutton)

        buttonsizer.Realize()
        sizer.Add(buttonsizer, flag=wx.ALIGN_RIGHT | wx.ALL, border=10)

        sizer.Fit(self)
        self.SetAutoLayout(True)
        self.SetSizer(sizer)

    def OnSave(self, evt):
        '''
        Saves debug log to file.
        '''
        dlg = wx.FileDialog(
            self,
            _('Save debug log as...'),
            os.getcwd(),
            'wammu.log',
            '',
            wx.SAVE | wx.OVERWRITE_PROMPT | wx.CHANGE_DIR
        )
        if dlg.ShowModal() == wx.ID_OK:
            Wammu.ErrorLog.SaveLog(filename=dlg.GetPath())

    def OnSearch(self, evt):
        '''
        Opens search for simmilar problems in browser.
        '''
        Wammu.Webbrowser.Open(BUG_SEARCH_URL % self.traceid)

    def OnReport(self, evt):
        '''
        Opens web browser with bug reporting page.
        '''
        Wammu.Webbrowser.Open(BUG_REPORT_URL)



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Error log handling
'''

import Wammu
import tempfile
import wx
import sys
import os
import locale
if Wammu.gammu_error is None:
    import gammu

# set later in Wammu.Main to have correct debug filename
DEBUG_LOG_FILENAME = None

# Template for system information
SYSTEM_TEMPLATE = """
--------------- System information ----------------
Platform     %s
Python       %s
wxPython     %s
Wammu        %s
python-gammu %s
Gammu        %s
Bluetooth    %s
locales      %s (%s)
"""

def GetSystemInfo():
    """
    Returns system information in text form.
    """
    pyver = sys.version.split()[0]
    wxver = wx.VERSION_STRING
    wammuver = Wammu.__version__
    try:
        gammuver, pgammuver = gammu.Version()
    except:
        try:
            gammuver, pgammuver, ignore = gammu.Version()
        except:
            gammuver, pgammuver = ('Unknown', 'Unknown')
    loc, charset = locale.getdefaultlocale()
    bluez = 'None'
    try:
        import bluetooth
        bluez = 'PyBluez'
    except ImportError:
        pass

    result = SYSTEM_TEMPLATE % (
        sys.platform,
        pyver,
        wxver,
        wammuver,
        pgammuver,
        gammuver,
        bluez,
        loc,
        charset)

    if Wammu.configuration is not None:
        section = Wammu.configuration.ReadInt('/Gammu/Section')
        config = Wammu.configuration.gammu.GetConfig(section)
        result += 'connection   %s\n' % config['Connection']
        result += 'device       %s\n' % config['Device']
        result += 'model        %s\n' % config['Model']
    return result

def SaveLog(outf=None, filename=None):
    """
    Saves debug log to filename or handle. If none specified
    """
    if DEBUG_LOG_FILENAME is None:
        return None, None
    if outf is None:
        if filename is None:
            handle, name = tempfile.mkstemp('.log', 'wammu-crash-')
            outf = os.fdopen(handle, 'w+')
        else:
            name = filename
            outf = open(filename, 'w+')

    inf = open(DEBUG_LOG_FILENAME, 'r')
    outf.write(GetSystemInfo())
    outf.write(inf.read())
    inf.close()
    if filename is not None:
        outf.close()
    return outf, name



# -*- coding: UTF-8 -*-
#
# Copyright © 2003 - 2015 Michal Čihař <michal@cihar.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# vim: expandtab sw=4 ts=4 sts=4:
'''
Wammu - Phone manager
Image displaying classes to be embdeded inside wxHTML
'''

import wx
import wx.lib.throbber
import base64

defaultbmp = [
    '20 20 2 1',
    '. c Black',
    '  c None',
    '                    ',
    ' ..              .. ',
    ' ...            ... ',
    '  ...          ...  ',
    '   ...        ...   ',
    '    ...      ...    ',
    '     ...    ...     ',
    '      ...  ...      ',
    '       ......       ',
    '        ....        ',
    '        ....        ',
    '       ......       ',
    '      ...  ...      ',
    '     ...    ...     ',
    '    ...      ...    ',
    '   ...        ...   ',
    '  ...          ...  ',
    ' ...            ... ',
    ' ..              .. ',
    '                    ']

class MemoryInputStream(wx.InputStream):
    def __init__(self, data):
        import cStringIO
        wx.InputStream.__init__(self, cStringIO.StringIO(data))

class EncodedBitmap(wx.StaticBitmap):
    def __init__(self, parent, tooltip='Image', image=None, size=None, scale=1):
        if image is None:
            image = defaultbmp
        image = wx.ImageFromStream(MemoryInputStream(base64.b64decode(image)))
        if scale > 1:
            bitmap = wx.BitmapFromImage(image.Scale(image.GetWidth() * scale, image.GetHeight() * scale))
        else:
            bitmap = wx.BitmapFromImage(image)
        wx.StaticBitmap.__init__(self, parent, -1, bitmap, (0, 0))
        self.SetToolTipString(tooltip)

class Bitmap(wx.StaticBitmap):
    def __init__(self, parent, tooltip='Image', image=None, size=None, scale=1):
        if image is None:
            image = defaultbmp
        bitmap = wx.BitmapFromXPMData(image)
        if scale > 1:
            img = wx.ImageFromBitmap(bitmap)
            bitmap = wx.BitmapFromImage(img.Scale(bitmap.GetWidth() * scale, bitmap.GetHeight() * scale))
        wx.StaticBitmap.__init__(self, parent, -1, bitmap, (0, 0))
        self.SetToolTipString(tooltip)

class Throbber(wx.lib.throbber.Throbber):
    def __init__(self, parent, tooltip='Animation', images=None, size=None, scale=1, delay=0.1):
        if images is None:
            images = [defaultbmp]
        bitmaps = []
        for im in images:
            bitmap = wx.BitmapFromXPMData(im)
            if scale > 1:
                img = wx.ImageFromBitmap(bitmap)
                bitmap = wx.BitmapFromImage(img.Scale(bitmap.GetWidth() * scale, bitmap.GetHeight() * scale))
            bitmaps.append(bitmap)
        wx.lib.throbber.Throbber.__init__(self, parent, -1, bitmaps, frameDelay=delay)
        self.SetToolTipString(tooltip)
